#!/usr/bin/perl
#
# IPCop CGIs
#
# This code is distributed under the terms of the GPL
#
# Copyright (c) 2006-2008 by Stephan Feddersen
# Co-Author Frank Mainz
#
# $Id: wio.pl, ver. 5.8 2008/12/05 s.feddersen Exp $
#

use strict;
use Time::HiRes qw(gettimeofday tv_interval);
use GD;
use Net::Ping;
use RRDs;
use Fatal qw/ open /;

require '/var/ipcop/general-functions.pl';
require "${General::swroot}/lang.pl";
require "/var/ipcop/wio/wiofunc.pl";

my $version = '5.8';
my ($debug, $result) = '';
my ($time, $start) = 0;
my $k  = 1;

my $owner     = getpwnam "nobody";
my $group     = getgrnam "nobody";
my $ipadrfile = "/var/log/wio/wioips";

if ( ! -e '/home/httpd/html/graphs/wio' ) { system( '/var/ipcop/wio/rewrite', 'graphs' ); }
if ( ! -e $ipadrfile ) { print ( "Die Datei $ipadrfile ist nicht vorhanden!\n" ); exit; }

foreach (@ARGV) {
  if ( $_ eq '-d' || $_ eq '--debug') { $debug = 1; }
  if ( $_ eq '-h' || $_ eq '--help' ) { die help(); }
  if ( $_ eq '-r' || $_ eq '--repair') { $result = repair(); printf "Die Datei $ipadrfile $result\n"; exit; }
}

my %wiosettings  = ();
my %mainsettings = ();

&General::readhash( "${General::swroot}/main/settings",     \%mainsettings );
&General::readhash( "${General::swroot}/ethernet/settings", \%wiosettings );
&General::readhash( "/var/log/wio/wiocfg",				    \%wiosettings );

my $mailclient   = "${General::swroot}/wio/sendEmail";
my $logfile      = "/var/log/wio/wiomail.log";
my $mailtxt      = "/var/log/wio/wiomail.txt";
my $logging      = $wiosettings{'LOGGING'};
my $mailen       = $wiosettings{'MAIL'};
my $smtpserver   = $wiosettings{'MAILSERVER'};
my $smtpport     = $wiosettings{'MAILSMTP'};
my $from_address = $wiosettings{'MAILFROM'};
my $username     = $wiosettings{'MAILNAME'};
my $password     = $wiosettings{'MAILPW'};
my $to_address1  = $wiosettings{'MAILTO1'};
my $to_address2  = $wiosettings{'MAILTO2'};
my $to_address3  = $wiosettings{'MAILTO3'};
my $timeout  = $wiosettings{'TIMEOUT'};
my $subject      = $Lang::tr{'wio_sub'};
my $imagedir     = "/home/httpd/html/graphs/wio";
my $rrdimgdir    = "$imagedir/rrdimg";
my $rrddir       = "/var/log/rrd/wio";
my $onoffip      = "/var/log/wio/wioscip";
my $redactive    = "/var/ipcop/red/active";
my $vpnip        = "$mainsettings{'HOSTNAME'}.$mainsettings{'DOMAINNAME'}";
my $redip        = $vpnip;
my $vpnpid       = ( -e "/var/run/pluto.pid" ? `awk '{print $1}' /var/run/pluto.pid`: '');
my $ovpnpid      = ( -e  "/var/run/openvpn.pid" ? `awk '{print $1}' /var/run/openvpn.pid`: '');
my ($ivpn, $tvpn)        = '';

my @befor = split(/ /, `ls -l --time-style=full-iso $ipadrfile`);

if ( -e "$redactive" ) {
	$redip = (($wiosettings{'RED_ADDRESS'} ne '' && $wiosettings{'RED_ADDRESS'} ne '0.0.0.0')?$wiosettings{'RED_ADDRESS'}:&General::FetchPublicIp());
}

my ( $gd, $red, $blue, $bgcolor, $f, $h, $w, $xpos, $ypos, $image );
my $height   = 15;
my ( $ipadr, $host, $enable, $remark, $dyndns, $mail, $mailon, $mailoff, $ping, $on );
my $i_ping   = 'icmp';
my $t_ping   = 'tcp';
my ($line, $msg, $msgcolor, $logmsg, $client, $mode, $onbak) = '';
my ( $ping_i, $ping_t, $ping_ib, $ping_tb, $ping_iv,  $ping_tv, $pingmode );

my $steptime   = $wiosettings{'CRON'} *= 60;
my ($newwrite, $togglestat)   = 0;

my @tmp      = '';
my @myarray  = '';
my @myfile   = '';
my @status   = '';

if ($debug) {
	$result = repair();
	printf "
Who Is Online? Ver. $version

Die Datei $ipadrfile $result\n";

	if ( $result eq "ist defekt!" ) {  exit -1; }

	$start = [gettimeofday];
	startdebug();
}

use Encode 'from_to';
my %tr = ();

if ( ${Lang::language} =~ /^(ar|bg|el|fa|gu|ja|ru|th|ur|vi|zh|zt)$/ ) {
	eval `/bin/cat "${General::swroot}/langs/en.pl"`;
}
else {
	%tr = %Lang::tr;
}

if (   ( ${Lang::language} eq 'cs' )
	|| ( ${Lang::language} eq 'hu' )
	|| ( ${Lang::language} eq 'pl' )
	|| ( ${Lang::language} eq 'sk' ) )
{
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-2" );
	}
}
elsif ( ${Lang::language} eq 'tr' ) {
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-9" );
	}
}
else {
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-1" );
	}
}

if ( -e "$onoffip" ) {
	open( FILE, "< $onoffip" );
	@myarray = <FILE>;
	close(FILE);
}
else {
	open( FILE, "< $ipadrfile" );
	@myarray = <FILE>;
	close(FILE);
}

###################################################################################################
# ping all clients
###################################################################################################

$f    = GD::Font->Small;
$f    = gdMediumBoldFont;
$h    = $height;
$h    = $f->height + 2 if ( $h < ( $f->height + 2 ) );
$w    = $f->width * 5 + 78;
$xpos = 5;
$ypos = ( $height - $f->height ) / 2;

foreach (@myarray) {
	chomp;
	@tmp = split( /\,/, $_ );

	($ipadr,$host,$enable,$remark,$dyndns,$mailon,$mailoff,$ping,$on) = @tmp;

	if ( $enable ne 'on' ) {
		push( @status, "$ipadr,$host,$enable,$remark,$dyndns,$mailon,$mailoff,$ping,$on\n" );
		next;
	}

	if ( -e "$redactive" && ( defined($dyndns) && ( $dyndns =~ 'on' ))) {
		my @fetchdyndnsip = gethostbyname($host);

		if ( $fetchdyndnsip[0] ne '' ) {
			@fetchdyndnsip = map ( &Socket::inet_ntoa($_), @fetchdyndnsip[4..$#fetchdyndnsip]);

			if ($fetchdyndnsip[0] ne $ipadr) {
				$ipadr = $fetchdyndnsip[0];
				$newwrite = 1;
			}
		}
	}

	$ping_i = $ping_t = $ping_ib = $ping_tb = $ping_iv =  $ping_tv = $pingmode = 0;
	
	my $i = Net::Ping->new( $i_ping, $timeout );
	unless ( defined $i ) { die "Can't create Net::Ping object $!"; }

	my $t = Net::Ping->new( $t_ping, $timeout );
	unless ( defined $t ) { die "Can't create Net::Ping object $!"; }

	my $ib = Net::Ping->new( $i_ping, $timeout );
	unless ( defined $ib ) { die "Can't create Net::Ping object $!"; }
	$ib->bind($redip);

	my $tb = Net::Ping->new( $t_ping, $timeout );
	unless ( defined $tb ) { die "Can't create Net::Ping object $!"; }
	$tb->bind($redip);

	$client = ( ( $dyndns eq 'on' || $ping eq 'fqdn' ) ? $host : $ipadr );

	if ($ovpnpid || $vpnpid)
	{
		$ivpn = Net::Ping->new( $i_ping, $timeout );
		unless ( defined $ivpn ) { die "Can't create Net::Ping object $!"; }
		$ivpn->bind($vpnip);

		$tvpn = Net::Ping->new( $t_ping, $timeout );
		unless ( defined $tvpn ) { die "Can't create Net::Ping object $!"; }
		$tvpn->bind($vpnip);
	}

	if ($debug) { 
		printf "%2s.  %15s", $k++,  ($client ne $ipadr ? $ipadr : $client );
		$time = [gettimeofday];
	}

	if ( ($ping_i =  $i->ping($client))
		|| ($ping_t = $t->ping($client))
		|| ($ping_ib = $ib->ping($client))
		|| ($ping_tb = $tb->ping($client))
		|| ($ovpnpid?($ping_iv = $ivpn->ping($client)) : 0)
		|| ($vpnpid?($ping_tv = $tvpn->ping($client)) : 0) )
	{
		$mode       = 100;
		$red        = "51";
		$blue       = "153";
		$msg        = "$Lang::tr{'wio up'}";
		$onbak      = $on;
		$togglestat = ( $on ne 'on' ) ? 1 : 0;
		$on         = 'on';
	}
	else {
		$mode       = 0;
		$red        = "153";
		$blue       = "51";
		$msg        = "$Lang::tr{'wio down'}";
		$onbak      = $on;
		$togglestat = ( $on ne 'off' ) ? 1 : 0;
		$on         = 'off';
	}

	push( @status, "$ipadr,$host,$enable,$remark,$dyndns,$mailon,$mailoff,$ping,$on\n" );

	if ($debug) {
		
		$mail = '----';
		if ( $mailon eq 'on' && $togglestat == 1 && $mode == 100 ) { $mail = 'Online'; }
		if ( $mailoff eq 'on' && $togglestat == 1 && $mode == 0 ) { $mail = 'Offline'; }
		if ( $dyndns ne 'on' ) { $dyndns = 'off'; }

		$pingmode = $ping_i ? 'ICMP' : $ping_t ? 'TCP' : $ping_ib ? 'ICMP+Bind' : $ping_tb ? 'TCP+Bind' : $ping_iv ? 'VPN ICMP' : $ping_tv ? 'VPN TCP' : 'No echo';  
		printf "%7s%7s%10s%10s   %.4f sek%12s\n", $ping, $dyndns, $msg, $mail, tv_interval($time), $pingmode;
	}
	
	$logmsg = "IP: $ipadr - Host: $host - Status: $msg";

	if ( $logging eq 'on' ) { system( '/usr/bin/logger', '-t', 'wio', $logmsg ); }

	if ( $dyndns eq 'on' || $ping ne 'ip' ) { unlink "$imagedir/updown-$host.png"; }
	else { unlink "$imagedir/updown-$ipadr.png"; }

	if ( $togglestat == 1 ) {

		$gd = new GD::Image( $w, $h );
		$msgcolor = $gd->colorAllocate( 204,  204,   204 );
		$bgcolor  = $gd->colorAllocate( $red, $blue, 51 );
		$gd->fill( 0, 0, $bgcolor );
		$gd->string( $f, $xpos, $ypos, $msg, $msgcolor );
		$image = $gd->png();

		$newwrite = 1;

		if ( $onbak ne '' && $mailen eq 'on' )	{
			if (( $mailon eq 'on' && $on eq 'on' ) || ( $mailoff eq 'on' && $on eq 'off' )) {
				system("/bin/date '+%a, %d.%m.%Y %H:%M:%S' >> $mailtxt");
				system("/bin/echo $logmsg >> $mailtxt");
				system("/bin/echo >> $mailtxt");
			}
		}

		open( IM, "> $imagedir/updown-$client.png" );
		binmode IM;
		print IM $image;
		close(IM);
	}

	if ( !-e "$imagedir/updown-$client.png" ) {
	
		$gd = new GD::Image( $w, $h );
		$msgcolor = $gd->colorAllocate( 204,  204,   204 );
		$bgcolor  = $gd->colorAllocate( $red, $blue, 51 );
		$gd->fill( 0, 0, $bgcolor );
		$gd->string( $f, $xpos, $ypos, $msg, $msgcolor );
		$image = $gd->png();
		
		open( IM, "> $imagedir/updown-$client.png" );
		binmode IM;
		print IM $image;
		close(IM);
	}

	undef $image;

	if ( $ping ne 'fqdn'  ) { $client = $host; }

	updatewiodata("$client");

	if ( -e "$rrddir/$client.rrd" ) {
		updatewiograph( "$client", "day" );
		updatewiograph( "$client", "week" );
		updatewiograph( "$client", "month" );
		updatewiograph( "$client", "year" );
	}

	$i->close();
	$t->close();
	$ib->close();
	$tb->close();

	if ( -e $ovpnpid || -e $vpnpid) { $ivpn->close(); $tvpn->close(); }
	
}

####################################################################################################
# write adressfile new
####################################################################################################
if ( !-e $onoffip && $newwrite == 1 ) {
	open( FILE, "> $ipadrfile" );
	flock FILE, 2;
	print FILE @status;
	close(FILE);
}
elsif ( -e $onoffip && $newwrite == 1 ) {
	system("/bin/sed -i 's#$tmp[0],$tmp[1],$tmp[2],$tmp[3],$tmp[4],$tmp[5],$tmp[6],$tmp[7],$tmp[8]#$ipadr,$host,$enable,$remark,$dyndns,$mailon,$mailoff,$ping,$on#g' $ipadrfile");
	chmod ( 0644, $ipadrfile ); 
	chown ( $owner, $group, $ipadrfile );
}

if ($debug) {
	my @after = split(/ /, `ls -l --time-style=full-iso $ipadrfile`);

	printf ("\nInformationen zur Datei $ipadrfile:

%9sBesitzer | Gruppe | Zeitstempel |      Datum
-----------------------------------------------------
 Vorher: %8s | %6s | %11s | %5s
Nachher: %8s | %6s | %11s | %5s\n\n"," ",$befor[2], $befor[3], substr($befor[6],0,8), substr($after[5],8,2).".".substr($befor[5],5,2).".".substr($befor[5],0,4), 
$after[2] , $after[3], substr($after[6],0,8) , substr($after[5],8,2).".".substr($after[5],5,2).".".substr($after[5],0,4));

printf ("Scriptlaufzeit:  %.4f sek\n\n", tv_interval($start));
}

if ( -e "$mailtxt" && $mailen eq 'on' ) {

	my $mailcall = "$mailclient -q -l $logfile -f $from_address -t $to_address1 -u $subject -o message-file=/var/log/wio/wiomail.txt -s $smtpserver:$smtpport";

	if ( $to_address2 ne '' ) { $mailcall .= " -t $to_address2"; }
	if ( $to_address3 ne '' ) { $mailcall .= " -t $to_address3"; }
	if ( $username ne '' ) { $mailcall .= " -xu $username"; }
	if ( $password ne '' ) { $mailcall .= " -xp '$password'"; }

	system($mailcall);
	system("/bin/rm -fr $mailtxt");
}

@tmp      = '';
@myarray  = '';
@status   = '';

if ( -e $onoffip ) { unlink($onoffip); }

sub updatewiodata {
	my $client = $_[0];

	if ( !-e "$rrddir/$client.rrd" ) {
		RRDs::create(
			"$rrddir/$client.rrd",      "--step=$steptime",
			"DS:mode:GAUGE:3600:0:100", "RRA:AVERAGE:0.5:1:576",
			"RRA:AVERAGE:0.5:6:672",    "RRA:AVERAGE:0.5:24:732",
			"RRA:AVERAGE:0.5:144:1460"
		);
		my $ERROR = RRDs::error;
		print "Error in RRD::create for Who Is Online: $ERROR\n" if $ERROR;
	}

	RRDs::update( "$rrddir/$client.rrd", "-t", "mode", "N:$mode" );

	my $error = RRDs::error;

	if ($error) {
		system( '/usr/bin/logger', '-t', 'wio', $error );
	}
}

sub updatewiograph {
	my $client = $_[0];
	my $period = $_[1];

	RRDs::graph(
		"$rrdimgdir/$client-$period.png",
		"--start",
		"-1$period",
		"-aPNG",
		"-i",
		"--alt-y-grid",
		"-w 600",
		"-h 100",
		"--color",
		"SHADEA#EAE9EE",
		"--color",
		"SHADEB#EAE9EE",
		"--color",
		"BACK#EAE9EE",
		"-t $client - $tr{'graph per'} $tr{$period}",
		"DEF:mode=$rrddir/$client.rrd:mode:AVERAGE",
		"CDEF:online=mode,UN,0,mode,IF,50,GT,100,0,IF",
		"CDEF:offline=mode,UN,100,mode,IF,50,LT,100,0,IF",
		"AREA:online#00FF00:$client $tr{'wio up'}\\j",
		"AREA:offline#FF0000:$client $tr{'wio down'}\\j"
	);
	my $ERROR = RRDs::error;
	print "Error in RRD::create for Who Is Online: $ERROR\n" if $ERROR;

	my $error = RRDs::error;

	if ($error) { system( '/usr/bin/logger', '-t', 'wio', $error ); }
}

sub startdebug {
printf "
HOSTNAME: \t$vpnip
TIMEOUT: \t$timeout Sekunde(n)
RED IP: \t$redip
";
	if ($ovpnpid) {printf "OVPN PID: \t$ovpnpid"}
	if ($vpnpid) {printf "VPN PID: \t$vpnpid"}
printf "
%3s%17s%7s%8s%9s%10s%13s%12s
-------------------------------------------------------------------------------
","Nr.", "IP-Adresse", "Ping", "DynDNS", "Status", "Mail", "Antwortzeit", "Antwort";
}

sub help {
	return "
Who Is Online? Ver. $version

This code is distributed under the terms of the GPL

Copyright (c) 2006-2008 by Stephan Feddersen
Co-Author Frank Mainz

use option -d for more runtime details
use option -r for repair adressfile\n";
}
 
 sub repair {
 	my $test = &wiofunc::check($ipadrfile); 
 	return ( $test eq 0 ? "ist in Ordnung!" : $test eq 1 ? "wurde aktualisiert!" : "ist defekt!" );
 } 