#!/usr/bin/perl
#
# IPCop CGIs
#
# This code is distributed under the terms of the GPL
#
# Copyright (c) 2006-2008 by Stephan Feddersen 
#
# $Id: wio.cgi, ver. 5.8 2008/12/05 s.feddersen Exp $
#

use strict;
use Socket;
use POSIX qw(strftime);
use Net::Ping;
use File::Copy;
use IO::Socket;
use LWP::UserAgent;
use Fatal qw/ open /;

# enable only the following on debugging purpose
#use warnings;
#use CGI::Carp 'fatalsToBrowser';

require '/var/ipcop/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";
require "${General::swroot}/wio/wiofunc.pl";

my %mainsettings    = ();
my %wiosettings     = ();
my %cgiparams       = ();
my %netsettings     = ();
my %vpnconfighash   = ();
my %ovpnconfighash  = ();
my %ovpnsettings    = ();
my %checked         = ();
my %selected        = ();

&General::readhash("${General::swroot}/main/settings", \%mainsettings);
&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);

if ( -e "/var/ipcop/xtiface/xtiface.conf" )
{
&General::readhash("${General::swroot}/xtiface/xtiface.conf", \%netsettings);
}

if ( ! -e '/home/httpd/html/graphs/wio' )
{
system ('/var/ipcop/wio/rewrite', 'graphs');
}

my $owner = getpwnam "nobody";
my $group = getgrnam "nobody";

my $addonversion = '5.8';
my $addonname    = 'Who Is Online?';
my $addonupdate  = "/var/log/wio/.update";

my $ipadrfile    = "/var/log/wio/wioips";
my $onoffip      = "/var/log/wio/wioscip";
my $wiosettings  = "/var/log/wio/wiocfg";
my $dyndnsfile   = "${General::swroot}/ddns/config";
my $importfile   = "/var/log/wio/importfile";
my $editfile     = "/var/log/wio/editfile";
my $mailclient   = "${General::swroot}/wio/sendEmail";
my $logfile      = "/var/log/wio/wiomail.log";
my $imagedir     = "/home/httpd/html/graphs/wio";
my $imgstatic    = "/images/wio";
my $rrddir       = "/var/log/rrd/wio";
my $rrdimgdir    = "/home/httpd/html/graphs/wio/rrdimg";
my $rrdimggraphs = "/graphs/wio/rrdimg";
my $refreshbox   = '<meta http-equiv="refresh" content="2;url=/cgi-bin/wio.cgi">';
my $refreshpage  = '<meta http-equiv="pragma" content="no-cache">';
my $errormessage = '';

my $mail_errormessage = '';
my $backup_errormessage = '',
my $import_errormessage = '';
my $import_csv_errormessage = '';
my $import_hosts_errormessage = '';
my $import_fixleases_errormessage = '';
my $import_arpscan_errormessage = '';

my $arp_msg      = '';
my $infomessage  = '';
my $delete       = '';
my $host         = '';
my $ipadr        = '';
my $on           = '';
my $remark       = '';
my $dyndns       = '';
my $sendemailon  = '';
my $sendemailoff = '';
my $pingmethode  = '';
my $online       = '';
my $result       = '';
my $sortstring   = '^IPADR|^HOST|^REMARK';
my $showcount    = 0;

my $ovpnpid      = "/var/run/openvpn.pid";
my $ovpnrwip     = ();
my $ovpnrwmask   = ();
my $ovpnn2nip    = (); 
my $ovpnn2nmask  = (); 
my @ovpnstatus   = ();
my @ovpnconfig   = ();

my $vpnpid       = "/var/run/pluto.pid";
my $dotip        = ();
my $vpnn2nip     = ();
my $vpnn2nmask   = ();
my $vpnrwip      = ();
my $vpnrwmask    = ();
my @vpnstatus    = ();
my @vpnconfig    = ();
my @vpnipsec     = ();

my $fqdn         = "$mainsettings{'HOSTNAME'}.$mainsettings{'DOMAINNAME'}";
my $redactive    = "/var/ipcop/red/active";
my $redip        = '';
my $redhostname  = '';

my @tmpfilename  = ();
my @dates        = ();
my @ipaddresses  = ();
my @names        = ();
my @remark       = ();
my @filenames    = ();
my @current      = ();
my @ddns         = ();
my @arpcache     = ();
my @arpadd       = ();
my @nosaved      = ('IPADR','HOST','REMARK','DYNDNS','SENDEMAILON','SENDEMAILOFF','PINGMETHODE','MODEHELP','ONLINE');

############################################################################################################################
# language work around
############################################################################################################################

use Encode 'from_to';
my %tr=();

if (${Lang::language} =~ /^(ar|bg|el|fa|gu|ja|ru|th|ur|vi|zh|zt)$/)
{
	eval `/bin/cat "${General::swroot}/langs/en.pl"`;
}
else
{
	%tr=%Lang::tr;
}

if (   ( ${Lang::language} eq 'cs' )
	|| ( ${Lang::language} eq 'hu' )
	|| ( ${Lang::language} eq 'pl' )
	|| ( ${Lang::language} eq 'sk' ) )
{
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-2" );
	}
}
elsif ( ${Lang::language} eq 'tr' ) {
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-9" );
	}
}
else {
	foreach my $key ( keys %Lang::tr ) {
		from_to( $tr{$key}, "utf-8", "iso-8859-1" );
	}
}

############################################################################################################################
# some color settings and color work arounds
############################################################################################################################

my @dummy = ( ${Header::colourblue}, ${Header::colourorange}, ${Header::table1colour} );

undef (@dummy);

############################################################################################################################
# generate some pngs
############################################################################################################################

if ( ! -e "$imagedir/wionoimage.png" )
{
	&wiofunc::noimagepng();
}

if ( -e "$redactive" )
{
	$redip = &General::FetchPublicIp();
	$redhostname = (gethostbyaddr(pack("C4", split (/\./, $redip)), 2))[0];
	&wiofunc::wanpng();
}
else
{
	$redip = '-';
	$redhostname = '-';
	&wiofunc::wanpng();
}

############################################################################################################################
# generate vpn pngs
############################################################################################################################

if ( -e "$vpnpid" )
{
	if ( ! -e "$imagedir/wiovpnclosed.png" )
	{
		&wiofunc::vpnstatuspng();
	}

	&General::readhasharray("${General::swroot}/vpn/config", \%vpnconfighash);

	@vpnstatus = `/usr/sbin/ipsec auto --status`;
	@vpnconfig = `/bin/cat /var/ipcop/vpn/config`;
	@vpnipsec = `/bin/cat /proc/net/ipsec_eroute`;
}
else
{
	if ( -e "$imagedir/wiovpnclosed.png" )
	{ 
		unlink "$imagedir/wiovpnclosed.png";
	}

	if ( -e "$imagedir/wiovpnopen.png" )
	{ 
		unlink "$imagedir/wiovpnopen.png";
	}

	if ( -e "$imagedir/wiovpnoff.png" )
	{ 
		unlink "$imagedir/wiovpnoff.png";
	}
}

############################################################################################################################
# generate openvpn pngs
############################################################################################################################

if ( -e "$ovpnpid" )
{
	if ( ! -e "$imagedir/wioovpnclosed.png" )
	{
		&wiofunc::ovpnstatuspng();
	}

	&General::readhasharray("${General::swroot}/ovpn/ovpnconfig", \%ovpnconfighash);
	&General::readhash("${General::swroot}/ovpn/settings", \%ovpnsettings);

	if ( -e "/var/log/ovpnserver.log" )
	{
		@ovpnstatus = `/bin/cat /var/log/ovpnserver.log`;
	}
	else
	{
		@ovpnstatus = `/bin/cat /var/ipcop/ovpn/server.log`;
	}

	@ovpnconfig = `/bin/cat /var/ipcop/ovpn/ovpnconfig`;
}
else
{
	if ( -e "$imagedir/wioovpnclosed.png" )
	{
		unlink "$imagedir/wioovpnclosed.png";
	}

	if ( -e "$imagedir/wioovpnopen.png" )
	{
		unlink "$imagedir/wioovpnopen.png";
	}

	if ( -e "$imagedir/wioovpnoff.png" )
	{
		unlink "$imagedir/wioovpnoff.png";
	}
}

############################################################################################################################
# some wio settings
############################################################################################################################

$wiosettings{'ACTION'} = '';
$wiosettings{'COUNT'} = '';
$wiosettings{'ID'} = '';
$wiosettings{'SORT'} = 'IPADR';
$wiosettings{'HOST'} = '';
$wiosettings{'IPADR'} = '';
$wiosettings{'EN'} = 'on';
$wiosettings{'REMARK'} = '';
$wiosettings{'DYNDNS'} = '';
$wiosettings{'SENDEMAILON'} = '';
$wiosettings{'SENDEMAILOFF'} = '';
$wiosettings{'PINGMETHODE'} = '';
$wiosettings{'MODEHELP'} = '';
$wiosettings{'TIMEOUT'} = '1';
$wiosettings{'ONLINE'} = '-';
$wiosettings{'CRON'} = '15';
$wiosettings{'ENABLE'} = 'off';
$wiosettings{'UPDATE'} = 'off';
$wiosettings{'LOGGING'} = 'off';
$wiosettings{'GREENIPLOW'} = '';
$wiosettings{'GREENIPHIGH'} = '';
$wiosettings{'BLUEIPLOW'} = '';
$wiosettings{'BLUEIPHIGH'} = '';
$wiosettings{'ORANGEIPLOW'} = '';
$wiosettings{'ORANGEIPHIGH'} = '';
$wiosettings{'GRAY1IPLOW'} = '';
$wiosettings{'GRAY1IPHIGH'} = '';
$wiosettings{'GRAY2IPLOW'} = '';
$wiosettings{'GRAY2IPHIGH'} = '';
$wiosettings{'GRAY3IPLOW'} = '';
$wiosettings{'GRAY3IPHIGH'} = '';
$wiosettings{'GRAY4IPLOW'} = '';
$wiosettings{'GRAY4IPHIGH'} = '';
$wiosettings{'MAIL'} = 'off';
$wiosettings{'MAILSERVER'} = '';
$wiosettings{'MAILSMTP'} = '25';
$wiosettings{'MAILFROM'} = '';
$wiosettings{'MAILNAME'} = '';
$wiosettings{'MAILPW'} = '';
$wiosettings{'MAILTO1'} = '';
$wiosettings{'MAILTO2'} = '';
$wiosettings{'MAILTO3'} = '';

&Header::getcgihash(\%wiosettings);
&Header::getcgihash(\%mainsettings);
&Header::getcgihash(\%netsettings);

############################################################################################################################
# read some informations
############################################################################################################################

open(FILE, "$ipadrfile");
flock FILE, 2;
@current = <FILE>;
close (FILE);

open(FILE, "$dyndnsfile");
flock FILE, 2;
@ddns = <FILE>;
close (FILE);

############################################################################################################################
# get network ips
############################################################################################################################

if ( $wiosettings{'ACTION'} ne 'wio_run_green' )
{
	my $green_network = "$netsettings{'GREEN_NETADDRESS'}"; 
	my @greenipnumbers = split (/\./, $green_network); 
	$wiosettings{'GREENIPLOW'} = sprintf("%d.%d.%d.%d", $greenipnumbers[0], $greenipnumbers[1], $greenipnumbers[2], $greenipnumbers[3]+1);

	my $green_broadcast = "$netsettings{'GREEN_BROADCAST'}"; 
	my @greenbroadcast = split (/\./, $green_broadcast); 
	$wiosettings{'GREENIPHIGH'} = sprintf("%d.%d.%d.%d", $greenbroadcast[0], $greenbroadcast[1], $greenbroadcast[2], $greenbroadcast[3]-1);
}

if ( $wiosettings{'ACTION'} ne 'wio_run_blue' )
{
	if ( defined($netsettings{'BLUE_DEV'}) && defined($netsettings{'BLUE_ADDRESS'}) )
	{
		my $blue_network = "$netsettings{'BLUE_NETADDRESS'}"; 
		my @blueipnumbers = split (/\./, $blue_network); 
		$wiosettings{'BLUEIPLOW'} = sprintf("%d.%d.%d.%d", $blueipnumbers[0], $blueipnumbers[1], $blueipnumbers[2], $blueipnumbers[3]+1); 

		my $blue_broadcast = "$netsettings{'BLUE_BROADCAST'}"; 
		my @bluebroadcast = split (/\./, $blue_broadcast); 
		$wiosettings{'BLUEIPHIGH'} = sprintf("%d.%d.%d.%d", $bluebroadcast[0], $bluebroadcast[1], $bluebroadcast[2], $bluebroadcast[3]-1); 
	}
}

if ( $wiosettings{'ACTION'} ne 'wio_run_orange' )
{
	if ( defined($netsettings{'ORANGE_DEV'}) && defined($netsettings{'ORANGE_ADDRESS'}) )
	{
		my $orange_network = "$netsettings{'ORANGE_NETADDRESS'}"; 
		my @orangeipnumbers = split (/\./, $orange_network); 
		$wiosettings{'ORANGEIPLOW'} = sprintf("%d.%d.%d.%d", $orangeipnumbers[0], $orangeipnumbers[1], $orangeipnumbers[2], $orangeipnumbers[3]+1); 

		my $orange_broadcast = "$netsettings{'ORANGE_BROADCAST'}"; 
		my @orangebroadcast = split (/\./, $orange_broadcast); 
		$wiosettings{'ORANGEIPHIGH'} = sprintf("%d.%d.%d.%d", $orangebroadcast[0], $orangebroadcast[1], $orangebroadcast[2], $orangebroadcast[3]-1); 
	}
}

if ( $wiosettings{'ACTION'} ne 'wio_run_gray1' )
{
	if ( defined($netsettings{'GRAY1_DEV'}) && $netsettings{'GRAY1_DEV'} ne 'disabled' )
	{
		my $gray1_network = "$netsettings{'GRAY1_NETADDRESS'}"; 
		my @gray1ipnumbers = split (/\./, $gray1_network); 
		$wiosettings{'GRAY1IPLOW'} = sprintf("%d.%d.%d.%d", $gray1ipnumbers[0], $gray1ipnumbers[1], $gray1ipnumbers[2], $gray1ipnumbers[3]+1);

		my $gray1_broadcast = "$netsettings{'GRAY1_BROADCAST'}"; 
		my @gray1broadcast = split (/\./, $gray1_broadcast); 
		$wiosettings{'GRAY1IPHIGH'} = sprintf("%d.%d.%d.%d", $gray1broadcast[0], $gray1broadcast[1], $gray1broadcast[2], $gray1broadcast[3]-1);
	}
}

if ( $wiosettings{'ACTION'} ne 'wio_run_gray2' )
{
	if ( defined($netsettings{'GRAY2_DEV'}) && $netsettings{'GRAY2_DEV'} ne 'disabled' )
	{
		my $gray2_network = "$netsettings{'GRAY2_NETADDRESS'}"; 
		my @gray2ipnumbers = split (/\./, $gray2_network); 
		$wiosettings{'GRAY2IPLOW'} = sprintf("%d.%d.%d.%d", $gray2ipnumbers[0], $gray2ipnumbers[1], $gray2ipnumbers[2], $gray2ipnumbers[3]+1);

		my $gray2_broadcast = "$netsettings{'GRAY2_BROADCAST'}"; 
		my @gray2broadcast = split (/\./, $gray2_broadcast); 
		$wiosettings{'GRAY2IPHIGH'} = sprintf("%d.%d.%d.%d", $gray2broadcast[0], $gray2broadcast[1], $gray2broadcast[2], $gray2broadcast[3]-1);
	}
}

if ( $wiosettings{'ACTION'} ne 'wio_run_gray3' )
{
	if ( defined($netsettings{'GRAY3_DEV'}) && $netsettings{'GRAY3_DEV'} ne 'disabled' )
	{
		my $gray3_network = "$netsettings{'GRAY3_NETADDRESS'}"; 
		my @gray3ipnumbers = split (/\./, $gray3_network); 
		$wiosettings{'GRAY3IPLOW'} = sprintf("%d.%d.%d.%d", $gray3ipnumbers[0], $gray3ipnumbers[1], $gray3ipnumbers[2], $gray3ipnumbers[3]+1);

		my $gray3_broadcast = "$netsettings{'GRAY3_BROADCAST'}"; 
		my @gray3broadcast = split (/\./, $gray3_broadcast); 
		$wiosettings{'GRAY3IPHIGH'} = sprintf("%d.%d.%d.%d", $gray3broadcast[0], $gray3broadcast[1], $gray3broadcast[2], $gray3broadcast[3]-1);
	}
}

if ( $wiosettings{'ACTION'} ne 'wio_run_gray4' )
{
	if ( defined($netsettings{'GRAY4_DEV'}) && $netsettings{'GRAY4_DEV'} ne 'disabled' )
	{
		my $gray4_network = "$netsettings{'GRAY4_NETADDRESS'}"; 
		my @gray4ipnumbers = split (/\./, $gray4_network); 
		$wiosettings{'GRAY4IPLOW'} = sprintf("%d.%d.%d.%d", $gray4ipnumbers[0], $gray4ipnumbers[1], $gray4ipnumbers[2], $gray4ipnumbers[3]+1);

		my $gray4_broadcast = "$netsettings{'GRAY4_BROADCAST'}"; 
		my @gray4broadcast = split (/\./, $gray4_broadcast); 
		$wiosettings{'GRAY4IPHIGH'} = sprintf("%d.%d.%d.%d", $gray4broadcast[0], $gray4broadcast[1], $gray4broadcast[2], $gray4broadcast[3]-1);
	}
}

############################################################################################################################
# save wio settings
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_save'}.'1' )
{
	if ( $wiosettings{'MAIL'} eq 'on' )
	{
		if ($wiosettings{'MAILTO1'} eq '' && $wiosettings{'MAILTO2'} eq '' && $wiosettings{'MAILTO3'} eq '' )
		{
			$mail_errormessage = $Lang::tr{'wio_mail_error_nomailto'};
		}

		if ($wiosettings{'MAILFROM'} eq '')
		{
			$mail_errormessage = $Lang::tr{'wio_mail_error_nomailfrom'};
		}
		
		if ($wiosettings{'MAILSERVER'} eq '')
		{
			$mail_errormessage = $Lang::tr{'wio_mail_error_nomailserver'};
		}

		if ($wiosettings{'MAILSMTP'} eq '')
		{
			$mail_errormessage = $Lang::tr{'wio_mail_error_nomailsmtp'};
		}
		
		if ($wiosettings{'MAILTO1'} ne '')
		{
			unless(&General::validemail($wiosettings{'MAILTO1'}))
			{
				$mail_errormessage = $Lang::tr{'wio_mail_error'};
			}
		}

		if ($wiosettings{'MAILTO2'} ne '')
		{
			unless(&General::validemail($wiosettings{'MAILTO2'}))
			{
				$mail_errormessage = $Lang::tr{'wio_mail_error'};
			}
		}

		if ($wiosettings{'MAILTO3'} ne '')
		{
			unless(&General::validemail($wiosettings{'MAILTO3'}))
			{
				$mail_errormessage = $Lang::tr{'wio_mail_error'};
			}
		}

		if ($wiosettings{'MAILFROM'} ne '')
		{
			unless(&General::validemail($wiosettings{'MAILFROM'}))
			{
				$mail_errormessage = $Lang::tr{'wio_mail_error'};
			}
		}
	}

	unless ($mail_errormessage)
	{
		$cgiparams{'SORT'} = $wiosettings{'SORT'};
		$cgiparams{'CRON'} = $wiosettings{'CRON'};
		$cgiparams{'ENABLE'} = $wiosettings{'ENABLE'};
		$cgiparams{'UPDATE'} = $wiosettings{'UPDATE'};
		$cgiparams{'LOGGING'} = $wiosettings{'LOGGING'};
		$cgiparams{'TIMEOUT'} = $wiosettings{'TIMEOUT'};
		$cgiparams{'ACTION'} = $wiosettings{'ACTION'};
		
		if ( $wiosettings{'MAIL'} eq 'on' )
		{
			$cgiparams{'MAIL'} = $wiosettings{'MAIL'};
			$cgiparams{'MAILSERVER'} = $wiosettings{'MAILSERVER'};
			$cgiparams{'MAILSMTP'} = $wiosettings{'MAILSMTP'};
			$cgiparams{'MAILFROM'} = $wiosettings{'MAILFROM'};
			$cgiparams{'MAILNAME'} = $wiosettings{'MAILNAME'};
			$cgiparams{'MAILPW'} = $wiosettings{'MAILPW'};
			$cgiparams{'MAILTO1'} = $wiosettings{'MAILTO1'};
			$cgiparams{'MAILTO2'} = $wiosettings{'MAILTO2'};
			$cgiparams{'MAILTO3'} = $wiosettings{'MAILTO3'};
		}
		else
		{
			$cgiparams{'MAIL'} = $wiosettings{'MAIL'};
			$cgiparams{'MAILSERVER'} = '';
			$cgiparams{'MAILSMTP'} = '';
			$cgiparams{'MAILFROM'} = '';
			$cgiparams{'MAILNAME'} = '';
			$cgiparams{'MAILPW'} = '';
			$cgiparams{'MAILTO1'} = '';
			$cgiparams{'MAILTO2'} = '';
			$cgiparams{'MAILTO3'} = '';
		}

		&General::writehash("$wiosettings", \%cgiparams);
		system ('/var/ipcop/wio/rewrite', 'start');
		&General::readhash("$wiosettings", \%wiosettings);
	}

	if ($wiosettings{'ENABLE'} eq 'off')
	{
		open(FILE, "> $logfile");
		flock FILE, 2;
		print FILE $delete;
		close(FILE);

		open(FILE, "> $ipadrfile");
		flock FILE, 2;
		print FILE $delete;
		close(FILE);

		unlink glob "$rrddir/*";
		unlink glob "$rrdimgdir/*";
		unlink glob "$imagedir/updown*";
		unlink "$imagedir/wionoimage.png";
		unlink "$imagedir/wiowan.png";

		if ( -e "$imagedir/wiovpnclosed.png" ) { unlink "$imagedir/wiovpnclosed.png"; }
		if ( -e "$imagedir/wiovpnopen.png" ) { unlink "$imagedir/wiovpnopen.png"; }
		if ( -e "$imagedir/wiovpnoff.png" ) { unlink "$imagedir/wiovpnoff.png"; }
	
		if ( -e "$imagedir/wioovpnclosed.png" ) { unlink "$imagedir/wioovpnclosed.png"; }
		if ( -e "$imagedir/wioovpnopen.png" ) { unlink "$imagedir/wioovpnopen.png"; }
		if ( -e "$imagedir/wioovpnoff.png" ) { unlink "$imagedir/wioovpnoff.png"; }
	}

	if ($wiosettings{'ENABLE'} eq 'on')
	{
		system('/usr/bin/logger', '-t', 'wio', $Lang::tr{'wio_enabled_log'});
	} 
	else
	{
		system('/usr/bin/logger', '-t', 'wio', $Lang::tr{'wio_disable_log'});
	}
}

############################################################################################################################
# save imported clients
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_save'}.'2' )
{
	my $count = 0;
	my $error = '';

	while ( $count < $wiosettings{'COUNT'} )
	{
		if ( defined($wiosettings{"USE$count"}) && $wiosettings{"USE$count"} eq 'on' )
		{
			$wiosettings{'IPADR'} = $wiosettings{"IPADR$count"};
			$wiosettings{'HOST'} = $wiosettings{"HOST$count"};
			$wiosettings{'EN'} = $wiosettings{"EN$count"};
			$wiosettings{'REMARK'} = $wiosettings{"REMARK$count"};
			$wiosettings{'DYNDNS'} = $wiosettings{"DYNDNS$count"};
			$wiosettings{'SENDEMAILON'} = $wiosettings{"SENDEMAILON$count"};
			$wiosettings{'SENDEMAILOFF'} = $wiosettings{"SENDEMAILOFF$count"};
			$wiosettings{'PINGMETHODE'} = $wiosettings{"PINGMETHODE$count"};
			$wiosettings{'ONLINE'} = '-';
			$wiosettings{'MODEHELP'} = $wiosettings{"MODEHELP$count"};

			if ( ! (defined($wiosettings{'DYNDNS'})) )
			{
				if ( $wiosettings{'IPADR'} eq "" )
				{
					if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_ip_empty'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_ip_empty'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_ip_empty'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_ip_empty'}; $error = 'error'; }
				}
				else
				{
					unless(&General::validip($wiosettings{'IPADR'}))
					{
						if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_ip_error'}; $error = 'error'; }
						if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_ip_error'}; $error = 'error'; }
						if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_ip_error'}; $error = 'error'; }
						if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_ip_error'}; $error = 'error'; }
					}
				}
			}

			if ( $wiosettings{'HOST'} eq "" )
			{
				if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_host_empty'}; $error = 'error'; }
				if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_host_empty'}; $error = 'error'; }
				if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_host_empty'}; $error = 'error'; }
				if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_host_empty'}; $error = 'error'; }
			}
			else
			{
				unless(&General::validdomainname($wiosettings{'HOST'}))
				{
					if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_hostname_error'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_hostname_error'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_hostname_error'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_hostname_error'}; $error = 'error'; }
				}
			}

			if ( $wiosettings{'PINGMETHODE'} eq 'fqdn' )
			{
				unless(&General::validfqdn($wiosettings{'HOST'}))
				{
					if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error';	}
					if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
					if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
				}
			}
	
			if ( (defined($wiosettings{'DYNDNS'}) && $wiosettings{'DYNDNS'} eq 'on' ) )
			{
				if ( $wiosettings{'PINGMETHODE'} eq 'fqdn' || $wiosettings{'PINGMETHODE'} eq 'ip' )
				{
					unless(&General::validfqdn($wiosettings{'HOST'}))
					{
						if ( $wiosettings{'MODEHELP'} eq 'csv' ) { $import_csv_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error';	}
						if ( $wiosettings{'MODEHELP'} eq 'hosts' ) { $import_hosts_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
						if ( $wiosettings{'MODEHELP'} eq 'fixleases' ) { $import_fixleases_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
						if ( $wiosettings{'MODEHELP'} eq 'arpscan' ) { $import_arpscan_errormessage = $Lang::tr{'wio_fqdn_error'}; $error = 'error'; }
					}
				}

				$wiosettings{'IPADR'} = &wiofunc::getip($wiosettings{'HOST'});

				my $count = 1;
				my $dummy = '1.2.3';

				if ( ! defined($wiosettings{'IPADR'}) )
				{
					if ( -z "$ipadrfile" )
					{
						my $dummyip = $dummy . "." . $count;
						$wiosettings{'IPADR'} = $dummyip;
						$infomessage = $Lang::tr{'wio_dyndns_info'};
						$count++;
					}
					elsif ( -T "$ipadrfile" )
					{
						open(FILE, "$ipadrfile");
						flock FILE, 2;
						my @checktemp = <FILE>;
						close (FILE);

						foreach my $checkline (@checktemp)
						{
							my $dummyip = $dummy . "." . $count;
							my ( $ipadr,$host,$on,$remark,$dyndns,$sendemail,$pingmethode ) = split (/\,/, $checkline);

							if ( $ipadr eq $dummyip )
							{	
								$count++;
							}

							$wiosettings{'IPADR'} = $dummyip;
							$infomessage = $Lang::tr{'wio_dyndns_info'};
						}
					}
				}
			}

			$wiosettings{'REMARK'} =~ s/,/&#44;/g;

			if ( $error eq 'error' )
			{
				open(FILE, ">> $editfile");
				flock FILE, 2;
				print FILE "$wiosettings{'IPADR'},$wiosettings{'HOST'},$wiosettings{'EN'},$wiosettings{'REMARK'},$wiosettings{'DYNDNS'},$wiosettings{'SENDEMAILON'},$wiosettings{'SENDEMAILOFF'},$wiosettings{'PINGMETHODE'},$wiosettings{'ONLINE'}\n";
				close(FILE);
				
				undef ($error);
			}
			else
			{
				unshift (@current, "$wiosettings{'IPADR'},$wiosettings{'HOST'},$wiosettings{'EN'},$wiosettings{'REMARK'},$wiosettings{'DYNDNS'},$wiosettings{'SENDEMAILON'},,$wiosettings{'SENDEMAILOFF'}$wiosettings{'PINGMETHODE'},$wiosettings{'ONLINE'}\n");
				&SortDataFile;
			}
		}
	$count++;
	}

	map ($wiosettings{$_} = '' ,@nosaved);
	unlink ($importfile);
}

############################################################################################################################
# enable / disabel client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'enable disable client'} )
{
	chomp(@current[$wiosettings{'ID'}]);
	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$ipadr = $temp[0];
	$host = $temp[1];

	$temp[2] = $temp[2] ne '' ? '' : 'on';
	@current[$wiosettings{'ID'}] = join (',', @temp)."\n";
	$wiosettings{'ID'} = '';

	if ( $temp[2] ne 'on' )
	{
		if ( -e "$imagedir/updown-$ipadr.png" )
		{
			unlink "$imagedir/updown-$ipadr.png";
		}
		else
		{
			unlink "$imagedir/updown-$host.png";
		}
		unlink "$rrddir/$host.rrd";
		unlink glob "$rrdimgdir/$host-*";
	}

	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);
}

############################################################################################################################
# enable / disable dyndns
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'enable disable dyndns'} )
{
	chomp(@current[$wiosettings{'ID'}]);
	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$ipadr = $temp[0];
	$host = $temp[1];
	
	$temp[4] = $temp[4] ne '' ? '' : 'on';
	@current[$wiosettings{'ID'}] = join (',', @temp)."\n";
	$wiosettings{'ID'} = '';

	if ( $temp[4] eq 'on' )
	{
		unlink "$imagedir/updown-$ipadr.png";
	}
	else
	{
		unlink "$imagedir/updown-$host.png";
	}
	
	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);
}

############################################################################################################################
# enable / disable sendemailon
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_mail_online'} )
{
	chomp(@current[$wiosettings{'ID'}]);
	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$temp[5] = $temp[5] ne '' ? '' : 'on';
	
	@current[$wiosettings{'ID'}] = join (',', @temp)."\n";
	$wiosettings{'ID'} = '';

	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);
}

############################################################################################################################
# enable / disable sendemailoff
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_mail_offline'} )
{
	chomp(@current[$wiosettings{'ID'}]);
	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$temp[6] = $temp[6] ne '' ? '' : 'on';
	
	@current[$wiosettings{'ID'}] = join (',', @temp)."\n";
	$wiosettings{'ID'} = '';

	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);
}

############################################################################################################################
# add or update client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_client_add'} )
{
	if ( $wiosettings{'DYNDNS'} ne 'on' && $wiosettings{'IPADR'} eq "" )
	{
		$errormessage = $Lang::tr{'wio_ip_empty'};
		goto ERROR;
	}
	elsif ( $wiosettings{'IPADR'} ne "" )
	{
		unless(&General::validip($wiosettings{'IPADR'}))
		{
			$errormessage = $Lang::tr{'wio_ip_error'};
			goto ERROR;
		}
	}

	if ( $wiosettings{'HOST'} eq "" )
	{
		$errormessage = $Lang::tr{'wio_host_empty'};
		goto ERROR;
	}
	else
	{
		unless(&General::validdomainname($wiosettings{'HOST'}))
		{
			$errormessage = $Lang::tr{'wio_hostname_error'};
			goto ERROR;
		}
	}

	if ( $wiosettings{'PINGMETHODE'} eq 'fqdn' )
	{
		unless(&General::validfqdn($wiosettings{'HOST'}))
		{
			$errormessage = $Lang::tr{'wio_fqdn_error'};
			goto ERROR;
		}
	}
	
	if ( $wiosettings{'DYNDNS'} eq 'on' )
	{
		if ( $wiosettings{'PINGMETHODE'} eq 'fqdn' || $wiosettings{'PINGMETHODE'} eq 'ip' )
		{
			unless(&General::validfqdn($wiosettings{'HOST'}))
			{
				$errormessage = $Lang::tr{'wio_fqdn_error'};
				goto ERROR;
			}
		}
	
		$wiosettings{'IPADR'} = &wiofunc::getip($wiosettings{'HOST'});

		my $count = 1;
		my $dummy = '1.2.3';

		if ( $wiosettings{'IPADR'} eq "" )
		{
			if ( -z "$ipadrfile" )
			{
				my $dummyip = $dummy . "." . $count;
				$wiosettings{'IPADR'} = $dummyip;
				$infomessage = $Lang::tr{'wio_dyndns_info'};
				$count++;
			}
			elsif ( -T "$ipadrfile" )
			{
				open(FILE, "$ipadrfile");
				flock FILE, 2;
				my @checktemp = <FILE>;
				close (FILE);

				foreach my $checkline (@checktemp)
				{
					my $dummyip = $dummy . "." . $count;
					my ( $ipadr,$host,$on,$remark,$dyndns,$sendemailon,$sendemailoff,$pingmethode,$online ) = split (/\,/, $checkline);

					if ( $ipadr eq $dummyip )
					{	
						$count++;
					}
	
					$wiosettings{'IPADR'} = $dummyip;
					$infomessage = $Lang::tr{'wio_dyndns_info'};
				}
			}
		}

		$wiosettings{'PINGMETHODE'} = 'fqdn';
	}

	$wiosettings{'REMARK'} =~ s/,/&#44;/g;
	$wiosettings{'ONLINE'} = "$wiosettings{'ONLINE'}";
	
	open(FILE, "$ipadrfile");
	flock FILE, 2;
	my @checktemp = <FILE>;
	close (FILE);

	foreach my $checkaddup (@checktemp)
	{
		chomp($checkaddup);

		my @addup = split (/\,/, $checkaddup);
		my @host = split (/\./, $addup[1]);

		my $ipadr = $addup[0];
		my $hostname1 = $addup[1];
		my $hostname2 = $host[0];

		my @check = split (/\./, $wiosettings{'HOST'});

		if ((( $hostname1 eq $check[0] || $hostname2 eq $check[0] )) && $host[1] eq $check[1] )
		{	
			$errormessage = $Lang::tr{'wio_host_exists'};
			goto ERROR;
		}

		if ( $ipadr eq $wiosettings{'IPADR'} )
		{	
			$errormessage = $Lang::tr{'wio_ip_exists'};
			goto ERROR;
		}
	}

	unless ($errormessage)
	{
		unshift (@current, "$wiosettings{'IPADR'},$wiosettings{'HOST'},$wiosettings{'EN'},$wiosettings{'REMARK'},$wiosettings{'DYNDNS'},$wiosettings{'SENDEMAILON'},$wiosettings{'SENDEMAILOFF'},$wiosettings{'PINGMETHODE'},$wiosettings{'ONLINE'}\n");
		$wiosettings{'ID'} = '';
		map ($wiosettings{$_} = '' ,@nosaved);
		&SortDataFile;
	}
}

############################################################################################################################
# scan networks (green/blue/orange/gray1/gray2/gray3/gray4)
############################################################################################################################

if ((( $wiosettings{'ACTION'} eq 'wio_run_green' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_blue' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_orange' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_gray1' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_gray2' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_gray3' ) ||
	( $wiosettings{'ACTION'} eq 'wio_run_gray4' )) ||
	
	$import_arpscan_errormessage ne '' )
{
	my $greeniplow = "$wiosettings{'GREENIPLOW'}";
	my $greeniphigh = "$wiosettings{'GREENIPHIGH'}";

	my $blueiplow = "$wiosettings{'BLUEIPLOW'}";
	my $blueiphigh = "$wiosettings{'BLUEIPHIGH'}";

	my $orangeiplow = "$wiosettings{'ORANGEIPLOW'}";
	my $orangeiphigh = "$wiosettings{'ORANGEIPHIGH'}";

	my $gray1iplow = "$wiosettings{'GRAY1IPLOW'}";
	my $gray1iphigh = "$wiosettings{'GRAY1IPHIGH'}";
	
	my $gray2iplow = "$wiosettings{'GRAY2IPLOW'}";
	my $gray2iphigh = "$wiosettings{'GRAY2IPHIGH'}";
	
	my $gray3iplow = "$wiosettings{'GRAY3IPLOW'}";
	my $gray3iphigh = "$wiosettings{'GRAY3IPHIGH'}";
	
	my $gray4iplow = "$wiosettings{'GRAY4IPLOW'}";
	my $gray4iphigh = "$wiosettings{'GRAY4IPHIGH'}";

	if ( ! ( -e $editfile ) )
	{
		my $dev = '';
		my $iprange = '';

		if ( $wiosettings{'ACTION'} eq 'wio_run_orange' )
		{
			$dev = $netsettings{'ORANGE_DEV'}; 
			$iprange = $orangeiplow . "-" . $orangeiphigh; 
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_blue' )
		{
			$dev = $netsettings{'BLUE_DEV'}; 
			$iprange = $blueiplow . "-" . $blueiphigh;
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_green' )
		{
			$dev = $netsettings{'GREEN_DEV'}; 
			$iprange = $greeniplow . "-" . $greeniphigh; 
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_gray1' )
		{
			$dev = $netsettings{'GRAY1_DEV'}; 
			$iprange = $gray1iplow . "-" . $gray1iphigh;
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_gray2' )
		{
			$dev = $netsettings{'GRAY2_DEV'};
			$iprange = $gray2iplow . "-" . $gray2iphigh;
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_gray3' )
		{
			$dev = $netsettings{'GRAY3_DEV'};
			$iprange = $gray3iplow . "-" . $gray3iphigh;
		}
		elsif ( $wiosettings{'ACTION'} eq 'wio_run_gray4' )
		{
			$dev = $netsettings{'GRAY4_DEV'};
			$iprange = $gray4iplow . "-" . $gray4iphigh;
		}

		my @temp = `/var/ipcop/wio/arpscan -a -s $dev $iprange`;

		open(FILE, "> $importfile");
		flock FILE, 2;
		print FILE @temp;
		close(FILE);
	}

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, '');
&Header::openbigbox('100%', 'LEFT');

	if ($import_arpscan_errormessage)
	{
		&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
			print "<table width='100%'>";
			print "<tr><td><font class='base' color='#FF0000'>$import_arpscan_errormessage</font></td></tr>";
			print "</table>";
		&Header::closebox();
	}

my $count = 0;
my @arpfile = '';

if ( -e "$editfile" )
{
	open(FILE, "$editfile");
	flock FILE, 2;
	@arpfile = <FILE>;
	close (FILE);
}
else
{
	open(FILE, "$importfile");
	flock FILE, 2;
	@arpfile = <FILE>;
	close FILE;
}

	foreach (@arpfile)
	{
		chomp;
		
		my @line = split (/\,/, $_);

		if ( -e "$editfile" )
		{
			$wiosettings{'IPADR$count'} = $line[0];
			$wiosettings{'HOST$count'} = $line[1];
			$wiosettings{'REMARK$count'} = $line[3];
			$wiosettings{'DYNDNS$count'} = $line[4];
			$wiosettings{'SENDEMAILON$count'} = $line[5];
			$wiosettings{'SENDEMAILOFF$count'} = $line[6];
			$wiosettings{'PINGMETHODE$count'} = $line[7];

			$checked{'PINGMETHODE$count'}{'ip'} = '';
			$checked{'PINGMETHODE$count'}{'fqdn'} = '';
			$checked{'PINGMETHODE$count'}{$wiosettings{'PINGMETHODE$count'}} = "checked='checked'";

			$checked{'DYNDNS$count'}{'on'} = ($wiosettings{'DYNDNS$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILON$count'}{'on'} = ($wiosettings{'SENDEMAILON$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILOFF$count'}{'on'} = ($wiosettings{'SENDEMAILOFF$count'} eq '' ) ? '' : "checked='checked'";
		}
		else
		{
			$wiosettings{'IPADR$count'} = $line[1];
			$wiosettings{'HOST$count'} = gethostbyaddr(inet_aton($line[1]), AF_INET);
			$wiosettings{'REMARK$count'} = '';
			$wiosettings{'PINGMETHODE$count'} = 'ip';

			$checked{'PINGMETHODE$count'}{'ip'} = ($wiosettings{'PINGMETHODE$count'} eq '' ) ? '' : "checked='checked'";
		}

		$wiosettings{'EN$count'} = 'on';
		$wiosettings{'USE$count'} = 'on';
		$wiosettings{'MODEHELP$count'} = 'arpscan';

		$checked{'EN$count'}{'on'} = ($wiosettings{'EN$count'} eq '' ) ? '' : "checked='checked'";
		$checked{'USE$count'}{'on'} = ($wiosettings{'USE$count'} eq '' ) ? '' : "checked='checked'";

		open(FILE, "$ipadrfile");
		flock FILE, 2;
		my @checktemp = <FILE>;
		close (FILE);

		foreach my $checkline (@checktemp)
		{
			chomp($checkline);

			my @checkline = split (/\,/, $checkline);
			my @host = split (/\./, $checkline[1]);

			my $ipadr = $checkline[0];
			my $hostname1 = $checkline[1];
			my $hostname2 = $host[0];

			if ( $hostname1 eq $wiosettings{'HOST$count'} || $hostname2 eq $wiosettings{'HOST$count'} )
			{	
				$errormessage = $Lang::tr{'wio_host_exists'};
			}

			if ( $ipadr eq $wiosettings{'IPADR$count'} )
			{	
				$errormessage = $Lang::tr{'wio_ip_exists'};
			}
		}

unless ($errormessage)
{

$showcount++;

&Header::openbox('100%', 'LEFT', "$Lang::tr{'wio_import_data'}$wiosettings{'HOST$count'}$Lang::tr{'wio_import_data2'}");

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td>$Lang::tr{'wio_use'}</td>
	<td><input type='checkbox' name='USE$count' $checked{'USE$count'}{'on'} /></td>
</tr>
<tr>
	<td>$Lang::tr{'wio_client_enable'}</td>
	<td><input type='checkbox' name='EN$count' $checked{'EN$count'}{'on'} /></td>
</tr>
<tr>
END

if ( $wiosettings{'IPADR$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' /></td>
END
}

if ( $wiosettings{'HOST$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' /></td>
END
}

print <<END;
	<td>$Lang::tr{'remark'}:&nbsp;<img align='absmiddle' src='/blob.gif' alt='*' /></td>
	<td><input type='text' name='REMARK$count' value='$wiosettings{'REMARK$count'}' SIZE='18' /></td>
</tr>
<tr>
	<td class='base' nowrap>$Lang::tr{'wio_ping_send'}:</td>
	<td align='left' nowrap>IP&nbsp;<input type='radio' name='PINGMETHODE$count' value='ip' $checked{'PINGMETHODE$count'}{'ip'} />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;FQDN&nbsp;<input type='radio' name='PINGMETHODE$count' value='fqdn' $checked{'PINGMETHODE$count'}{'fqdn'} /></td>
	<td>$Lang::tr{'wio_dyndns'}:</td>
	<td><input type='checkbox' name='DYNDNS$count' $checked{'DYNDNS$count'}{'on'} /></td>
END
if ( $wiosettings{'MAIL'} eq 'on' )
{
	print"<td>$Lang::tr{'wio_sendemail'}:</td>";
	print"<td><input type='checkbox' name='SENDEMAILON$count' $checked{'SENDEMAILON$count'}{'on'} />&nbsp;$Lang::tr{'wio_online'}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='checkbox' name='SENDEMAILOFF$count' $checked{'SENDEMAILOFF$count'}{'on'} />&nbsp;$Lang::tr{'wio_offline'}</td>";
}
else
{
	print "<td>&nbsp;</td>";
	print "<td>&nbsp;</td>";
}
print <<END;
</tr>
</table>
<hr>
<table width='100%'>
	<td class='base' width='50%' align='left'><img align='absmiddle' src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
	<td><input type='hidden' name='MODEHELP$count' value='$wiosettings{'MODEHELP$count'}'></td>
</table>
END

&Header::closebox();
}

$errormessage = '';
$count++;
}

if ( $showcount gt 0 )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio_import_infos'});

print <<END;
<table width='100%'>
<tr>
	<td width='50%'><font color='#0000FF'>$Lang::tr{'wio_import_infos_csv'}</font></td>
	<td width='25%' align='center'><input type='hidden' name='COUNT' value='$count' /><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}2' /><input type='submit' value='$Lang::tr{'wio_save'}' /></form></td>
	<td width='25%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>
</tr>
</table>
END
	
&Header::closebox();
}
else
{
$infomessage = $Lang::tr{'wio_import_info_csv'};
&General::readhash("$wiosettings", \%wiosettings);

if ( -e "$importfile" )	{ unlink ($importfile); }
goto INFO;
}

&Header::closebigbox();
&Header::closepage();

if ( -e "$editfile" ) { unlink ($editfile); }
exit 0;
}

############################################################################################################################
# refresh wio status
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_refresh'} )
{

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, $refreshbox);
&Header::openbigbox('100%', 'LEFT', '');

&Header::openbox('100%', 'left', $Lang::tr{'wio_info'});

print <<END;
<table width='50%' align='center'>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td><font class='base' color='#0000FF'><b>$Lang::tr{'wio_msg'}</b></font></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='center'><img align='absmiddle' src='$imgstatic/wioreload.gif' /></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

&Header::closebox();

&Header::closebigbox();
&Header::closepage();

system ('/var/ipcop/wio/rewrite', 'update', '&');

while ( `ps -A | grep wio.pl` ) { }

exit 0;		
}

############################################################################################################################
# refresh dyndns ip
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_dyndns_refresh'} )
{
	if ( -e "$redactive" )
	{
		chomp(@current[$wiosettings{'ID'}]);

		my @temp = split (/\,/, @current[$wiosettings{'ID'}]);
		my @fetchdyndnsip = gethostbyname($temp[1]);		
		
		if ($fetchdyndnsip[0] ne '')
		{
			@fetchdyndnsip = map ( &Socket::inet_ntoa($_), @fetchdyndnsip[4..$#fetchdyndnsip] );
			$temp[0] = $fetchdyndnsip[0];
		}

		@current[$wiosettings{'ID'}] = join (',', @temp)."\n";
		$wiosettings{'ID'} = '';

		open(FILE, "> $ipadrfile");
		flock FILE, 2;
		print FILE @current;
		close(FILE);
	}
	else
	{
		$wiosettings{'ID'} = '';
	}
}

############################################################################################################################
# refresh single client status
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_onoff_refresh'} )
{
	my $ps = `ps -A | grep wio.pl`;

	if ( $ps ne '' )
	{ 
		$wiosettings{'ID'} = '';
		while ( `ps -A | grep wio.pl` ) { }
		goto END;
	}
	else
	{
		chomp(@current[$wiosettings{'ID'}]);

		my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

		open(FILE, "> $onoffip");
		flock FILE, 2;
		print FILE "$temp[0],$temp[1],$temp[2],$temp[3],$temp[4],$temp[5],$temp[6],$temp[7],$temp[8]";
		close(FILE);

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, $refreshbox);
&Header::openbigbox('100%', 'LEFT', '');

&Header::openbox('100%', 'left', $Lang::tr{'wio_info'});

print <<END;
<table width='50%' align='center'>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td><font class='base' color='#0000FF'><b>$Lang::tr{'wio_msg'}</b></font></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='center'><img align='absmiddle' src='$imgstatic/wioreload.gif' /></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

&Header::closebox();

&Header::closebigbox();
&Header::closepage();

system ('/var/ipcop/wio/rewrite', 'update', '&');

while ( `ps -A | grep wio.pl` ) { }

$wiosettings{'ID'} = '';

exit 0;		
}
END:
}

############################################################################################################################
# show client graphs
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_graphs'} )
{
	my $line = '';
	my @currentnew = ();

	open(FILE, "$ipadrfile");
	flock FILE, 2;
	my @checktemp = <FILE>;
	close (FILE);

	foreach $line (@checktemp)
	{
		my ( $ipadr,$host,$on,$remark,$dyndns,$sendemailon,$sendemailoff,$pingmethode,$online ) = split (/\,/, $line);

		if ( $on ne 'on' ) { next; }
		else { push (@currentnew,($line)); }
	}

	chomp(@currentnew[$wiosettings{'ID'}]);

	my @temp = split (/\,/, @currentnew[$wiosettings{'ID'}]);

	my $graphs = "$rrdimggraphs";

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, );
&Header::openbigbox('100%', 'LEFT', '');

&Header::openbox('100%', 'left', "$Lang::tr{'wio_graphs_stat'} $temp[1]");

if ( -e "$rrdimgdir/$temp[1]-day.png" )
{
	my $fileday = "$rrdimgdir/$temp[1]-day.png";
	my @infoday = stat($fileday);
	my $timeday = "$infoday[9]\n";
	my $fileday_date = strftime("%d.%m.%Y",localtime($timeday));
	my $fileday_time = strftime("%H:%M:%S",localtime($timeday));
	
print <<END;
<table width='100%' align='center'>
<tr>
	<td><center><b>$Lang::tr{'the statistics were last updated at'}: $fileday_date $Lang::tr{'wio_backup_time'} $fileday_time</b></center></td>
</tr>
<tr>
	<td align='center'><img src='$graphs/$temp[1]-day.png' align='center' border='0' /></td>
</tr>
<tr>
	<td align='center' height='35'><hr></td>
</tr>
END

	my $fileweek = "$rrdimgdir/$temp[1]-week.png";
	my @infoweek = stat($fileweek);
	my $timeweek = "$infoweek[9]\n";
	my $fileweek_date = strftime("%d.%m.%Y",localtime($timeweek));
	my $fileweek_time = strftime("%H:%M:%S",localtime($timeweek));

print <<END;
<tr>
	<td><center><b>$Lang::tr{'the statistics were last updated at'}: $fileweek_date $Lang::tr{'wio_backup_time'} $fileweek_time</b></center></td>
</tr>
<tr>
	<td align='center'><img src='$graphs/$temp[1]-week.png' align='center' border='0' /></td>
</tr>
<tr>
	<td align='center' height='35'><hr></td>
</tr>
END

	my $filemonth = "$rrdimgdir/$temp[1]-month.png";
	my @infomonth = stat($filemonth);
	my $timemonth = "$infomonth[9]\n";
	my $filemonth_date = strftime("%d.%m.%Y",localtime($timemonth));
	my $filemonth_time = strftime("%H:%M:%S",localtime($timemonth));

print <<END;
<tr>
	<td><center><b>$Lang::tr{'the statistics were last updated at'}: $filemonth_date $Lang::tr{'wio_backup_time'} $filemonth_time</b></center></td>
</tr>
<tr>
	<td align='center'><img src='$graphs/$temp[1]-month.png' align='center' border='0' /></td>
</tr>
<tr>
	<td align='center' height='35'><hr></td>
</tr>
END

	my $fileyear = "$rrdimgdir/$temp[1]-year.png";
	my @infoyear = stat($fileyear);
	my $timeyear = "$infoyear[9]\n";
	my $fileyear_date = strftime("%d.%m.%Y",localtime($timeyear));
	my $fileyear_time = strftime("%H:%M:%S",localtime($timeyear));

print <<END;
<tr>
	<td><center><b>$Lang::tr{'the statistics were last updated at'}: $fileyear_date $Lang::tr{'wio_backup_time'} $fileyear_time</b></center></td>
</tr>
<tr>
	<td align='center'><img src='$graphs/$temp[1]-year.png' align='center' border='0' /></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

}
else
{

print <<END;
<table width='100%' align='center'>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='center'>$Lang::tr{'no information available'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

}

&Header::closebox();

print <<END;
<table width='100%'>
<tr>
	<td width='25%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>
</tr>
</table>
END

&Header::closebigbox();
&Header::closepage();

$wiosettings{'ID'} = '';

exit 0;
}

############################################################################################################################
# edit client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'edit'} )
{
	chomp(@current[$wiosettings{'ID'}]);
		
	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$wiosettings{'IPADR'} = $temp[0];
	$wiosettings{'HOST'} = $temp[1];
	$wiosettings{'EN'} = $temp[2];
	$wiosettings{'REMARK'} = $temp[3];
	$wiosettings{'DYNDNS'} = $temp[4];
	$wiosettings{'SENDEMAILON'} = $temp[5];
	$wiosettings{'SENDEMAILOFF'} = $temp[6];
	$wiosettings{'PINGMETHODE'} = $temp[7];
	$wiosettings{'ONLINE'} = $temp[8];
	
	$ipadr = $temp[0];
	$host = $temp[1];
	$dyndns = $temp[4];
	
	if ( $dyndns eq 'on' )
	{
		unlink "$imagedir/updown-$host.png";
	}
	else
	{
		unlink "$imagedir/updown-$ipadr.png";
	}
	
	unlink "$rrddir/$host.rrd";
	unlink glob "$rrdimgdir/$host-*";
	
	splice (@current,$wiosettings{'ID'},1);

	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);
}

############################################################################################################################
# add arp client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_add'} )
{
	open(FILE, "/var/log/wio/.arpcache");
	flock FILE, 2;
	@arpadd = <FILE>;
	close (FILE);

	chomp(@arpadd[$wiosettings{'ID'}]);

	my @temp = split (/\,/, @arpadd[$wiosettings{'ID'}]);

	$wiosettings{'IPADR'} = $temp[0];
	$wiosettings{'HOST'} = $temp[1];
	$wiosettings{'EN'} = 'on';
	$wiosettings{'PINGMETHODE'} = 'ip';
	$wiosettings{'ONLINE'} = 'on';

	if ( $wiosettings{'IPADR'} eq "" )
	{
		$arp_msg = $Lang::tr{'wio_ip_empty'};
		goto ERROR;
	}
	else
	{
		unless(&General::validip($wiosettings{'IPADR'}))
		{
			$arp_msg = $Lang::tr{'wio_ip_error'};
			goto ERROR;
		}
	}

	if ( $wiosettings{'HOST'} eq "" )
	{
		$arp_msg = $Lang::tr{'wio_host_empty'};
		goto ERROR;
	}
	else
	{
		unless(&General::validdomainname($wiosettings{'HOST'}))
		{
			$arp_msg = $Lang::tr{'wio_hostname_error'};
			goto ERROR;
		}
	}

	open(FILE, "$ipadrfile");
	flock FILE, 2;
	my @checktemp = <FILE>;
	close (FILE);

	foreach my $checkaddup (@checktemp)
	{
		chomp($checkaddup);

		my @addup = split (/\,/, $checkaddup);
		my @host = split (/\./, $addup[1]);

		my $ipadr = $addup[0];
		my $hostname1 = $addup[1];
		my $hostname2 = $host[0];

		my @check = split (/\./, $wiosettings{'HOST'});

		if ((( $hostname1 eq $check[0] || $hostname2 eq $check[0] )) && $host[1] eq $check[1] )
		{	
			$arp_msg = $Lang::tr{'wio_host_exists'};
			$wiosettings{'ID'} = '';
			goto ERROR;
		}

		if ( $ipadr eq $wiosettings{'IPADR'} )
		{	
			$arp_msg = $Lang::tr{'wio_ip_exists'};
			$wiosettings{'ID'} = '';
			goto ERROR;
		}
	}

	unless ($arp_msg)
	{
		unshift (@current, "$wiosettings{'IPADR'},$wiosettings{'HOST'},$wiosettings{'EN'},$wiosettings{'REMARK'},$wiosettings{'DYNDNS'},$wiosettings{'SENDEMAILON'},$wiosettings{'SENDEMAILOFF'},$wiosettings{'PINGMETHODE'},$wiosettings{'ONLINE'}\n");
		$wiosettings{'ID'} = '';
		map ($wiosettings{$_} = '' ,@nosaved);
		&SortDataFile;
	}
}

############################################################################################################################
# remove client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'remove'} )
{
	chomp(@current[$wiosettings{'ID'}]);

	my @temp = split (/\,/, @current[$wiosettings{'ID'}]);

	$ipadr = $temp[0];
	$host = $temp[1];
	$dyndns = $temp[4];
	
	if ( $dyndns eq 'on' )
	{
		unlink "$imagedir/updown-$host.png";
	}
	else
	{
		unlink "$imagedir/updown-$ipadr.png";
	}
	
	unlink "$rrddir/$host.rrd";
	unlink glob "$rrdimgdir/$host-*";

	splice (@current,$wiosettings{'ID'},1);

	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE @current;
	close(FILE);

	$wiosettings{'ID'} = '';
}

############################################################################################################################
# remove all clients
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_delete'} )
{
	open(FILE, "> $ipadrfile");
	flock FILE, 2;
	print FILE $delete;
	close(FILE);

	unlink glob "$rrddir/*";
	unlink glob "$rrdimgdir/*";
	unlink glob "$imagedir/updown*";
}

############################################################################################################################
# clear client
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_client_clear'} )
{
	map ($wiosettings{$_} = '' ,@nosaved);
	$errormessage = '';
}

############################################################################################################################
# clear sendemail logfile
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_mail_log_clear'} )
{
	open(FILE, "> $logfile");
	flock FILE, 2;
	print FILE $delete;
	close(FILE);
}

############################################################################################################################
# back function
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_back'} )
{
	if ( -e "$importfile" ) { unlink ($importfile); }
}

############################################################################################################################
# import csv file
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_import'}.'1' || $import_csv_errormessage ne '' )
{
	unless($import_csv_errormessage)
	{
		&Header::getcgihash(\%wiosettings, {'wantfile' => 1, 'filevar' => 'CSVFILE'});

		if ($wiosettings{'CSVFILE'} eq '' )
		{
			$import_errormessage = $Lang::tr{'wio_no_file_selected'};
			goto ERROR;
		}

		if ($wiosettings{'CSVFILE'} =~ /[^a-z0-9A-Z\ \.\-\_\:\\]+/ )
		{
			$import_errormessage = $Lang::tr{'wio_no_csv_error'};
			goto ERROR;
		}

		if (!($wiosettings{'CSVFILE'} =~ /.csv$/))
		{
			$import_errormessage = $Lang::tr{'wio_no_csv'};
			goto ERROR;
		}

		if (copy($wiosettings{'CSVFILE'}, "/var/log/wio/importfile") != 1)
		{
			$import_errormessage = $!;
			goto ERROR;
		}
	}

&General::readhash("$wiosettings", \%wiosettings);

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, '');
&Header::openbigbox('100%', 'LEFT');

	if ($import_csv_errormessage)
	{
		&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
			print "<table width='100%'>";
			print "<tr><td><font class='base' color='#FF0000'>$import_csv_errormessage</font></td></tr>";
			print "</table>";
		&Header::closebox();
	}
	
my $count = 0;
my @csv = '';

if ( -e "$editfile" )
{
open(FILE, "$editfile");
flock FILE, 2;
@csv = <FILE>;
close (FILE);
}
else
{
open(FILE, "$importfile");
flock FILE, 2;
@csv = <FILE>;
close (FILE);
}

foreach (sort @csv)
{
	chomp;

	my @line = split (/\,/, $_);

	$wiosettings{'IPADR$count'} = $line[0];
	$wiosettings{'HOST$count'} = $line[1];
	$wiosettings{'EN$count'} = $line[2];
	$wiosettings{'REMARK$count'} = $line[3];
	$wiosettings{'DYNDNS$count'} = $line[4];
	$wiosettings{'SENDEMAILON$count'} = $line[5];
	$wiosettings{'SENDEMAILOFF$count'} = $line[6];
	$wiosettings{'PINGMETHODE$count'} = $line[7];
	$wiosettings{'USE$count'} = 'on';
	$wiosettings{'MODEHELP$count'} = 'csv';

	$checked{'EN$count'}{'on'} = ($wiosettings{'EN$count'} eq '' ) ? '' : "checked='checked'";
	$checked{'DYNDNS$count'}{'on'} = ($wiosettings{'DYNDNS$count'} eq '' ) ? '' : "checked='checked'";
	$checked{'SENDEMAILON$count'}{'on'} = ($wiosettings{'SENDEMAILON$count'} eq '' ) ? '' : "checked='checked'";
	$checked{'SENDEMAILOFF$count'}{'on'} = ($wiosettings{'SENDEMAILOFF$count'} eq '' ) ? '' : "checked='checked'";
	$checked{'USE$count'}{'on'} = ($wiosettings{'USE$count'} eq '' ) ? '' : "checked='checked'";
	
	$checked{'PINGMETHODE$count'}{'ip'} = '';
	$checked{'PINGMETHODE$count'}{'fqdn'} = '';
	$checked{'PINGMETHODE$count'}{$wiosettings{'PINGMETHODE$count'}} = "checked='checked'";
	
	open(FILE, "$ipadrfile");
	flock FILE, 2;
	my @checkcsv = <FILE>;
	close (FILE);

	foreach my $checkcsv (@checkcsv)
	{
		chomp($checkcsv);

		my @csv = split (/\,/, $checkcsv);
		my @host = split (/\./, $csv[1]);
	
		my $ipadr = $csv[0];
		my $hostname1 = $csv[1];
		my $hostname2 = $host[0];

		if ( $hostname1 eq $wiosettings{'HOST$count'} || $hostname2 eq $wiosettings{'HOST$count'} )
		{	
			$errormessage =  $Lang::tr{'wio_host_exists'};
		}
	
		if ( $ipadr eq $wiosettings{'IPADR$count'} )
		{	
			$errormessage =  $Lang::tr{'wio_ip_exists'};
		}
	}

unless ($errormessage)
{

$showcount++;

&Header::openbox('100%', 'LEFT', "$Lang::tr{'wio_import_data'}$wiosettings{'HOST$count'}$Lang::tr{'wio_import_data2'}");

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td>$Lang::tr{'wio_use'}</td>
	<td><input type='checkbox' name='USE$count' $checked{'USE$count'}{'on'} /></td>
</tr>
<tr>
	<td>$Lang::tr{'wio_client_enable'}</td>
	<td><input type='checkbox' name='EN$count' $checked{'EN$count'}{'on'} /></td>
</tr>
<tr>
END

if ( $wiosettings{'IPADR$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' /></td>
END
}

if ( $wiosettings{'HOST$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' /></td>
END
}

print <<END;
	<td>$Lang::tr{'remark'}:&nbsp;<img align='absmiddle' src='/blob.gif' alt='*' /></td>
	<td><input type='text' name='REMARK$count' value='$wiosettings{'REMARK$count'}' SIZE='18' /></td>
</tr>
<tr>
	<td class='base' nowrap>$Lang::tr{'wio_ping_send'}:</td>
	<td align='left' nowrap>IP&nbsp;<input type='radio' name='PINGMETHODE$count' value='ip' $checked{'PINGMETHODE$count'}{'ip'} />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;FQDN&nbsp;<input type='radio' name='PINGMETHODE$count' value='fqdn' $checked{'PINGMETHODE$count'}{'fqdn'} /></td>
	<td>$Lang::tr{'wio_dyndns'}:</td>
	<td><input type='checkbox' name='DYNDNS$count' $checked{'DYNDNS$count'}{'on'} /></td>
END
if ( $wiosettings{'MAIL'} eq 'on' )
{
	print"<td>$Lang::tr{'wio_sendemail'}:</td>";
	print"<td><input type='checkbox' name='SENDEMAILON$count' $checked{'SENDEMAILON$count'}{'on'} />&nbsp;$Lang::tr{'wio_online'}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='checkbox' name='SENDEMAILOFF$count' $checked{'SENDEMAILOFF$count'}{'on'} />&nbsp;$Lang::tr{'wio_offline'}</td>";
}
else
{
	print "<td>&nbsp;</td>";
	print "<td>&nbsp;</td>";
}
print <<END;
</tr>
</table>
<hr>
<table width='100%'>
	<td class='base' width='50%' align='left'><img align='absmiddle' src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
	<td><input type='hidden' name='MODEHELP$count' value='$wiosettings{'MODEHELP$count'}'></td>
</table>
END

&Header::closebox();
}

$errormessage = '';
$count++;
}

if ( $showcount gt 0 )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio_import_infos'});

print <<END;
<table width='100%'>
<tr>
	<td width='50%'><font color='#0000FF'>$Lang::tr{'wio_import_infos_csv'}</font></td>
	<td width='25%' align='center'><input type='hidden' name='COUNT' value='$count' /><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}2' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_save'}' /></form></td>
	<td width='25%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>
</tr>
</table>
END
	
&Header::closebox();
}
else
{
$infomessage = $Lang::tr{'wio_import_info_csv'};
&General::readhash("$wiosettings", \%wiosettings);
goto INFO;
}

&Header::closebigbox();
&Header::closepage();

if ( -e "$editfile" ) { unlink ($editfile); }
exit 0;
}

############################################################################################################################
# import hosts file
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_import'}.'2' || $import_hosts_errormessage ne '' )
{
	&General::readhash("$wiosettings", \%wiosettings);

	&Header::showhttpheaders();
	&Header::openpage($Lang::tr{'wio'}, 1, '');
	&Header::openbigbox('100%', 'LEFT');

	if ($import_hosts_errormessage)
	{
		&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
			print "<table width='100%'>";
			print "<tr><td><font class='base' color='#FF0000'>$import_hosts_errormessage</font></td></tr>";
			print "</table>";
		&Header::closebox();
	}
	
	my $count = 0;
	my @hosts = '';

	if ( -e "$editfile" )
	{
		open(FILE, "$editfile");
		flock FILE, 2;
		@hosts = <FILE>;
		close (FILE);
	}
	else
	{
		open(FILE, "/var/ipcop/main/hosts");
		flock FILE, 2;
		@hosts = <FILE>;
		close (FILE);
	}

	foreach (sort @hosts)
	{
		chomp;

		my @line = split (/\,/, $_);

		if ( -e "$editfile" )
		{
			$wiosettings{'IPADR$count'} = $line[0];
			$wiosettings{'HOST$count'} = $line[1];
			$wiosettings{'EN$count'} = $line[2];
			$wiosettings{'REMARK$count'} = $line[3];
			$wiosettings{'DYNDNS$count'} = $line[4];
			$wiosettings{'SENDEMAILON$count'} = $line[5];
			$wiosettings{'SENDEMAILOFF$count'} = $line[6];
			$wiosettings{'PINGMETHODE$count'} = $line[7];
			
			$checked{'PINGMETHODE$count'}{'ip'} = '';
			$checked{'PINGMETHODE$count'}{'fqdn'} = '';
			$checked{'PINGMETHODE$count'}{$wiosettings{'PINGMETHODE$count'}} = "checked='checked'";
			
			$checked{'DYNDNS$count'}{'on'} = ($wiosettings{'DYNDNS$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILON$count'}{'on'} = ($wiosettings{'SENDEMAILON$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILOFF$count'}{'on'} = ($wiosettings{'SENDEMAILOFF$count'} eq '' ) ? '' : "checked='checked'";
		}
		else
		{
			$wiosettings{'IPADR$count'} = $line[1];
			$wiosettings{'HOST$count'} = $line[2];
			$wiosettings{'EN$count'} = 'on';
			$wiosettings{'DOM$count'} = $line[3];
			$wiosettings{'REMARK$count'} = '';
			$wiosettings{'PINGMETHODE$count'} = 'ip';
			
			$checked{'PINGMETHODE$count'}{'ip'} = ($wiosettings{'PINGMETHODE$count'} eq '' ) ? '' : "checked='checked'";
		}
		
		$wiosettings{'USE$count'} = 'on';
		$wiosettings{'MODEHELP$count'} = 'hosts';

		$checked{'EN$count'}{'on'} = ($wiosettings{'EN$count'} eq '' ) ? '' : "checked='checked'";
		$checked{'USE$count'}{'on'} = ($wiosettings{'USE$count'} eq '' ) ? '' : "checked='checked'";
	
		open(FILE, "$ipadrfile");
		flock FILE, 2;
		my @checkhosts = <FILE>;
		close (FILE);

		foreach my $checkhosts (@checkhosts)
		{
			chomp($checkhosts);
	
			my @hosts = split (/\,/, $checkhosts);
			my @host = split (/\./, $hosts[1]);
	
			my $ipadr = $hosts[0];
			my $hostname1 = $hosts[1];
			my $hostname2 = $host[0];

			if ( $hostname1 eq $wiosettings{'HOST$count'} || $hostname2 eq $wiosettings{'HOST$count'} )
			{	
				$errormessage =  $Lang::tr{'wio_host_exists'};
			}
			
			if ( $ipadr eq $wiosettings{'IPADR$count'} )
			{	
				$errormessage =  $Lang::tr{'wio_ip_exists'};
			}
		}

unless ($errormessage)
{

$showcount++;

&Header::openbox('100%', 'LEFT', "$Lang::tr{'wio_import_data'}$wiosettings{'HOST$count'}$Lang::tr{'wio_import_data2'}");

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>

<table width='100%'>
<tr>
	<td>$Lang::tr{'wio_use'}</td>
	<td><input type='checkbox' name='USE$count' $checked{'USE$count'}{'on'} /></td>
</tr>
<tr>
	<td>$Lang::tr{'wio_client_enable'}</td>
	<td><input type='checkbox' name='EN$count' $checked{'EN$count'}{'on'} /></td>
</tr>
<tr>
END

if ( $wiosettings{'IPADR$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' /></td>
END
}

if ( $wiosettings{'HOST$count'} eq '' )
{
if ( -e "$editfile" )
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}.$wiosettings{'DOM$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
}
else
{
if ( -e "$editfile" )
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}.$wiosettings{'DOM$count'}' SIZE='18' /></td>
END
}
}

print <<END;
	<td>$Lang::tr{'remark'}:&nbsp;<img align='absmiddle' src='/blob.gif' alt='*' /></td>
	<td><input type='text' name='REMARK$count' value='$wiosettings{'REMARK$count'}' size='18' /></td>
</tr>
<tr>
	<td class='base' nowrap>$Lang::tr{'wio_ping_send'}:</td>
	<td align='left' nowrap>IP&nbsp;<input type='radio' name='PINGMETHODE$count' value='ip' $checked{'PINGMETHODE$count'}{'ip'} />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;FQDN&nbsp;<input type='radio' name='PINGMETHODE$count' value='fqdn' $checked{'PINGMETHODE$count'}{'fqdn'} /></td>
	<td>$Lang::tr{'wio_dyndns'}:</td>
	<td><input type='checkbox' name='DYNDNS$count' $checked{'DYNDNS$count'}{'on'} /></td>
END
if ( $wiosettings{'MAIL'} eq 'on' )
{
	print"<td>$Lang::tr{'wio_sendemail'}:</td>";
	print"<td><input type='checkbox' name='SENDEMAILON$count' $checked{'SENDEMAILON$count'}{'on'} />&nbsp;$Lang::tr{'wio_online'}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='checkbox' name='SENDEMAILOFF$count' $checked{'SENDEMAILOFF$count'}{'on'} />&nbsp;$Lang::tr{'wio_offline'}</td>";
}
else
{
	print "<td>&nbsp;</td>";
	print "<td>&nbsp;</td>";
}
print <<END;
</tr>
</table>
<hr>
<table width='100%'>
	<td class='base' width='50%' align='left'><img align='absmiddle' src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
	<td><input type='hidden' name='MODEHELP$count' value='$wiosettings{'MODEHELP$count'}'></td>
</table>
END

&Header::closebox();
}

$errormessage = '';
$count++;
}

if ( $showcount gt 0 )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio_import_infos'});

print <<END;
<table width='100%'>
<tr>
	<td width='50%'><font color='#0000FF'>$Lang::tr{'wio_import_infos_csv'}</font></td>
	<td width='25%' align='center'><input type='hidden' name='COUNT' value='$count' /><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}2' /><input type='submit' value='$Lang::tr{'wio_save'}' /></form></td>
	<td width='25%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>
</tr>
</table>
END
	
&Header::closebox();
}
else
{
$infomessage = $Lang::tr{'wio_import_info_csv'};
&General::readhash("$wiosettings", \%wiosettings);
goto INFO;
}

&Header::closebigbox();
&Header::closepage();

if ( -e "$editfile" )	{ unlink ($editfile); }
exit 0;
}

############################################################################################################################
# import fixleases file
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_import'}.'3' || $import_fixleases_errormessage ne '' )
{
	&General::readhash("$wiosettings", \%wiosettings);

	&Header::showhttpheaders();
	&Header::openpage($Lang::tr{'wio'}, 1, '');
	&Header::openbigbox('100%', 'LEFT');

	if ($import_fixleases_errormessage)
	{
		&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
			print "<table width='100%'>";
			print "<tr><td><font class='base' color='#FF0000'>$import_fixleases_errormessage</font></td></tr>";
			print "</table>";
		&Header::closebox();
	}

	my $count = 0;
	my @fixleases = '';
	
	if ( -e "$editfile" )
	{
		open(FILE, "$editfile");
		flock FILE, 2;
		@fixleases = <FILE>;
		close (FILE);
	}
	else
	{	
		open(FILE, "/var/ipcop/dhcp/fixleases");
		flock FILE, 2;
		@fixleases = <FILE>;
		close (FILE);
	}
	
	foreach (sort @fixleases)
	{
		chomp;

		my @line = split (/\,/, $_);

		if ( -e "$editfile" )
		{
			$wiosettings{'IPADR$count'} = $line[0];
			$wiosettings{'HOST$count'} = $line[1];
			$wiosettings{'EN$count'} = $line[2];
			$wiosettings{'REMARK$count'} = $line[3];
			$wiosettings{'DYNDNS$count'} = $line[4];
			$wiosettings{'SENDEMAILON$count'} = $line[5];
			$wiosettings{'SENDEMAILOFF$count'} = $line[6];
			$wiosettings{'PINGMETHODE$count'} = $line[7];
			
			$checked{'PINGMETHODE$count'}{'ip'} = '';
			$checked{'PINGMETHODE$count'}{'fqdn'} = '';
			$checked{'PINGMETHODE$count'}{$wiosettings{'PINGMETHODE$count'}} = "checked='checked'";
			
			$checked{'DYNDNS$count'}{'on'} = ($wiosettings{'DYNDNS$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILON$count'}{'on'} = ($wiosettings{'SENDEMAILON$count'} eq '' ) ? '' : "checked='checked'";
			$checked{'SENDEMAILOFF$count'}{'on'} = ($wiosettings{'SENDEMAILOFF$count'} eq '' ) ? '' : "checked='checked'";
		}
		else
		{
			$wiosettings{'IPADR$count'} = $line[1];
			$wiosettings{'HOST$count'} = $line[7];
			$wiosettings{'EN$count'} = 'on';
			$wiosettings{'REMARK$count'} = $line[6];
			$wiosettings{'PINGMETHODE$count'} = 'ip';
			
			$checked{'PINGMETHODE$count'}{'ip'} = ($wiosettings{'PINGMETHODE$count'} eq '' ) ? '' : "checked='checked'";
		}
		
		$wiosettings{'USE$count'} = 'on';
		$wiosettings{'MODEHELP$count'} = 'fixleases';

		$checked{'EN$count'}{'on'} = ($wiosettings{'EN$count'} eq '' ) ? '' : "checked='checked'";
		$checked{'USE$count'}{'on'} = ($wiosettings{'USE$count'} eq '' ) ? '' : "checked='checked'";

		open(FILE, "$ipadrfile");
		flock FILE, 2;
		my @checkfixleases = <FILE>;
		close (FILE);

		foreach my $checkfixleases(@checkfixleases)
		{
			chomp($checkfixleases);
		
			my @fixleases = split (/\,/, $checkfixleases);
			my @host = split (/\./, $fixleases[1]);
			
			my $ipadr = $fixleases[0];
			my $hostname1 = $fixleases[1];
			my $hostname2 = $host[0];

			if ( $hostname1 eq $wiosettings{'HOST$count'} || $hostname2 eq $wiosettings{'HOST$count'} )
			{	
				$errormessage =  $Lang::tr{'wio_host_exists'};
			}
			
			if ( $ipadr eq $wiosettings{'IPADR$count'} )
			{	
				$errormessage =  $Lang::tr{'wio_ip_exists'};
			}
		}

unless ($errormessage)
{

$showcount++;

&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_import_leases'});

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td>$Lang::tr{'wio_use'}</td>
	<td><input type='checkbox' name='USE$count' $checked{'USE$count'}{'on'} /></td>
</tr>
<tr>
	<td>$Lang::tr{'wio_client_enable'}</td>
	<td><input type='checkbox' name='EN$count' $checked{'EN$count'}{'on'} /></td>
</tr>
<tr>
END

if ( $wiosettings{'IPADR$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR$count' value='$wiosettings{'IPADR$count'}' SIZE='18' /></td>
END
}

if ( $wiosettings{'HOST$count'} eq '' )
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' STYLE='background-color:#FFFF00' /></td>
END
}
else
{
print <<END;
	<td>$Lang::tr{'hostname'}:</td>
	<td><input type='text' name='HOST$count' value='$wiosettings{'HOST$count'}' SIZE='18' /></td>
END
}

print <<END;
	<td>$Lang::tr{'remark'}:&nbsp;<img align='absmiddle' src='/blob.gif' alt='*' /></td>
	<td><input type='text' name='REMARK$count' value='$wiosettings{'REMARK$count'}' SIZE='18' /></td>
</tr>
<tr>
	<td class='base' nowrap>$Lang::tr{'wio_ping_send'}:</td>
	<td align='left' nowrap>IP&nbsp;<input type='radio' name='PINGMETHODE$count' value='ip' $checked{'PINGMETHODE$count'}{'ip'} />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;FQDN&nbsp;<input type='radio' name='PINGMETHODE$count' value='fqdn' $checked{'PINGMETHODE$count'}{'fqdn'} /></td>
	<td>$Lang::tr{'wio_dyndns'}:</td>
	<td><input type='checkbox' name='DYNDNS$count' $checked{'DYNDNS$count'}{'on'} /></td>
END
if ( $wiosettings{'MAIL'} eq 'on' )
{
	print"<td>$Lang::tr{'wio_sendemail'}:</td>";
	print"<td><input type='checkbox' name='SENDEMAILON$count' $checked{'SENDEMAILON$count'}{'on'} />&nbsp;$Lang::tr{'wio_online'}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='checkbox' name='SENDEMAILOFF$count' $checked{'SENDEMAILOFF$count'}{'on'} />&nbsp;$Lang::tr{'wio_offline'}</td>";
}
else
{
	print "<td>&nbsp;</td>";
	print "<td>&nbsp;</td>";
}
print <<END;
</tr>
</table>
<hr>
<table width='100%'>
	<td class='base' width='50%' align='left'><img align='absmiddle' src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
	<td><input type='hidden' name='MODEHELP$count' value='$wiosettings{'MODEHELP$count'}'></td>
</table>
END

&Header::closebox();
}

$errormessage = '';
$count++;
}

if ( $showcount gt 0 )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio_import_infos'});

print <<END;
<table width='100%'>
<tr>
	<td width='50%'><font color='#0000FF'>$Lang::tr{'wio_import_infos_csv'}</font></td>
	<td width='25%' align='center'><input type='hidden' name='COUNT' value='$count' /><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}2' /><input type='submit' value='$Lang::tr{'wio_save'}' /></form></td>
	<td width='25%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>
</tr>
</table>
END
	
&Header::closebox();
}
else
{
$infomessage = $Lang::tr{'wio_import_info_csv'};
&General::readhash("$wiosettings", \%wiosettings);
goto INFO;
}

&Header::closebigbox();
&Header::closepage();

if ( -e "$editfile" ) { unlink ($editfile); }
exit 0;
}

############################################################################################################################
# create wio-backup.tar.gz
############################################################################################################################

if  ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_backup_create'} )
{
	if (system("/bin/tar -czf /var/log/wio/backup.tar.gz /var/log/wio/wiocfg /var/log/wio/wioips"))
	{
		$backup_errormessage = $Lang::tr{'wio_backup_error'};
		goto ERROR;
	}
	else
	{
		print "Content-type: application/gzip\n";
		print "Content-length: ";
		print (-s "/var/log/wio/backup.tar.gz");
		print "\n";
		print "Content-disposition: attachment; filename=wio-backup.tar.gz\n\n";

		open (FILE, "/var/log/wio/backup.tar.gz");
		flock FILE, 2;
		while (<FILE>) { print; }
		close (FILE);

		if (-e "/var/log/wio/backup.tar.gz")
		{
			unlink("/var/log/wio/backup.tar.gz");
		}

		exit;
	}
}

############################################################################################################################
# import wio-backup.tar.gz
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_backup_fileimport'} )
{
	&Header::getcgihash(\%wiosettings, {'wantfile' => 1, 'filevar' => 'BACKUPFILE'});

	if ($wiosettings{'BACKUPFILE'} eq '' )
	{
		$backup_errormessage = $Lang::tr{'wio_no_file_selected'};
		goto ERROR;
	}

	if ($wiosettings{'BACKUPFILE'} =~ /[^a-z0-9A-Z\ \.\-\_\:\\]+/ )
	{
		$backup_errormessage = $Lang::tr{'wio_no_csv_error'};
		goto ERROR;
	}

	if (!($wiosettings{'BACKUPFILE'} =~ /.tar.gz$/))
	{
		$backup_errormessage = $Lang::tr{'wio_backup_wrong_filetype'};
		goto ERROR;
	}

	if (copy($wiosettings{'BACKUPFILE'}, "/var/log/wio/backup.tar.gz") != 1)
	{
		$backup_errormessage = $!;
		goto ERROR;
	}

	my $exitcode = system("/bin/tar --no-same-owner --preserve-permissions -xzf /var/log/wio/backup.tar.gz -C /");
	if ($exitcode > 0)
	{
		$backup_errormessage = $Lang::tr{'wio_tar_error'};
		goto ERROR;
	}
	else
	{
		if ( (!(-e "/var/log/wio/wiocfg")) || (!(-e "/var/log/wio/wioips")) )
		{
			$backup_errormessage = $Lang::tr{'wio_invalid_restore_file'};
			goto ERROR;
		}
		else
		{
			system ('/var/ipcop/wio/rewrite', 'start');
			&wiofunc::check($ipadrfile);
			$infomessage = $Lang::tr{'wio_restore_success'};
		}
	}

	if (-e "/var/log/wio/backup.tar.gz")
	{
		unlink("/var/log/wio/backup.tar.gz");
	}
}

############################################################################################################################
# edit settings function
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'edit'}.'1' || $mail_errormessage ne '' )
{

unless($mail_errormessage) { &General::readhash("$wiosettings", \%wiosettings); }

$selected{'CRON'}{'5'} = '';
$selected{'CRON'}{'10'} = '';
$selected{'CRON'}{'15'} = '';
$selected{'CRON'}{'20'} = '';
$selected{'CRON'}{'25'} = '';
$selected{'CRON'}{'30'} = '';
$selected{'CRON'}{'35'} = '';
$selected{'CRON'}{'40'} = '';
$selected{'CRON'}{'45'} = '';
$selected{'CRON'}{'50'} = '';
$selected{'CRON'}{'55'} = '';
$selected{'CRON'}{$wiosettings{'CRON'}} = "selected='selected'";

$selected{'TIMEOUT'}{'1'} = '';
$selected{'TIMEOUT'}{'2'} = '';
$selected{'TIMEOUT'}{'3'} = '';
$selected{'TIMEOUT'}{'4'} = '';
$selected{'TIMEOUT'}{'5'} = '';
$selected{'TIMEOUT'}{'6'} = '';
$selected{'TIMEOUT'}{'7'} = '';
$selected{'TIMEOUT'}{'8'} = '';
$selected{'TIMEOUT'}{'9'} = '';
$selected{'TIMEOUT'}{'10'} = '';
$selected{'TIMEOUT'}{$wiosettings{'TIMEOUT'}} = "selected='selected'";

$checked{'ENABLE'}{'off'} = '';
$checked{'ENABLE'}{'on'} = '';
$checked{'ENABLE'}{$wiosettings{'ENABLE'}} = "checked='checked'";

$checked{'UPDATE'}{'off'} = '';
$checked{'UPDATE'}{'on'} = '';
$checked{'UPDATE'}{$wiosettings{'UPDATE'}} = "checked='checked'";

$checked{'LOGGING'}{'off'} = '';
$checked{'LOGGING'}{'on'} = '';
$checked{'LOGGING'}{$wiosettings{'LOGGING'}} = "checked='checked'";

$checked{'MAIL'}{'off'} = '';
$checked{'MAIL'}{'on'} = '';
$checked{'MAIL'}{$wiosettings{'MAIL'}} = "checked='checked'";

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, '');
&Header::openbigbox('100%', 'LEFT', '');

if ($mail_errormessage)
{
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#FF0000'>$mail_errormessage</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

&Header::openbox('100%', 'left', $Lang::tr{'wio settings'});

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_settings_msg'}</b></td>
	<td width='22%'>&nbsp;</td>
	<td width='22%'>&nbsp;</td>
	<td width='88%'>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio enabled'}</td>
	<td align='left'><input type='checkbox' name='ENABLE' $checked{'ENABLE'}{'on'} /></td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio cron'}</td>
	<td align='left'><select size='1' name='CRON' size='5'>
		<option $selected{'CRON'}{'5'}>5</option>
		<option $selected{'CRON'}{'10'}>10</option>
		<option $selected{'CRON'}{'15'}>15</option>
		<option $selected{'CRON'}{'20'}>20</option>
		<option $selected{'CRON'}{'25'}>25</option>
		<option $selected{'CRON'}{'30'}>30</option>
		<option $selected{'CRON'}{'35'}>35</option>
		<option $selected{'CRON'}{'40'}>40</option>
		<option $selected{'CRON'}{'45'}>45</option>
		<option $selected{'CRON'}{'50'}>50</option>
		<option $selected{'CRON'}{'55'}>55</option>
		</select>&nbsp;$Lang::tr{'wio min'}</td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio timeout'}</td>
	<td align='left'><select size='1' name='TIMEOUT' size='5'>
		<option $selected{'TIMEOUT'}{'1'}>1</option>
		<option $selected{'TIMEOUT'}{'2'}>2</option>
		<option $selected{'TIMEOUT'}{'3'}>3</option>
		<option $selected{'TIMEOUT'}{'4'}>4</option>
		<option $selected{'TIMEOUT'}{'5'}>5</option>
		<option $selected{'TIMEOUT'}{'6'}>6</option>
		<option $selected{'TIMEOUT'}{'7'}>7</option>
		<option $selected{'TIMEOUT'}{'8'}>8</option>
		<option $selected{'TIMEOUT'}{'9'}>9</option>
		<option $selected{'TIMEOUT'}{'10'}>10</option>
		</select>&nbsp;$Lang::tr{'wio sec'}</td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_update'}</td>
	<td align='left'><input type='checkbox' name='UPDATE' $checked{'UPDATE'}{'on'} /></td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_logging'}</td>
	<td align='left'><input type='checkbox' name='LOGGING' $checked{'LOGGING'}{'on'} /></td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mail_enabled'}</td>
	<td align='left'><input type='checkbox' name='MAIL' $checked{'MAIL'}{'on'}></td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailsender'}:</td>
	<td align='left'><input type='text' name='MAILFROM' value='$wiosettings{'MAILFROM'}' size='40' tabindex='1'></td>
	<td align='right'>$Lang::tr{'wio_mailserver'}:</td>
	<td align='left'><input type='text' name='MAILSERVER' value='$wiosettings{'MAILSERVER'}' size='25' tabindex='5'></td>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto1'}:</td>
	<td align='left'><input type='text' name='MAILTO1' value='$wiosettings{'MAILTO1'}' size='40' tabindex='2'></td>
	<td align='right'>$Lang::tr{'wio_mailsmtpport'}:</td>
	<td align='left'><input type='text' name='MAILSMTP' value='$wiosettings{'MAILSMTP'}' size='2' tabindex='6'></td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto2'}:</td>
	<td align='left'><input type='text' name='MAILTO2' value='$wiosettings{'MAILTO2'}' size='40' tabindex='3'></td>
	<td align='right'>$Lang::tr{'wio_mailserveruser'}:</td>
	<td align='left'><input type='text' name='MAILNAME' value='$wiosettings{'MAILNAME'}' size='25' tabindex='7'></td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto3'}:</td>
	<td align='left'><input type='text' name='MAILTO3' value='$wiosettings{'MAILTO3'}' size='40' tabindex='4'></td>
	<td align='right'>$Lang::tr{'wio_mailserverpw'}:</td>
	<td align='left'><input type='password' name='MAILPW' value='$wiosettings{'MAILPW'}' tabindex='8'></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td align='left'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}1' /><input type='submit' name='submit' value='$Lang::tr{'wio_save'}' />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td align='right'><small><a href="http://www.ipcop.h-loit.de" target="_blank"><font color='${Header::table1colour}'><b>$addonname Ver. $addonversion</b></font></a></small></td>
</tr>
</table>
</form>
END

&Header::closebox();

&Header::closebigbox();
&Header::closepage();

exit 0;
}

############################################################################################################################
# test sendemail function
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_mail_test'} )
{
	&General::readhash("$wiosettings", \%wiosettings);

	my $smtpserver = $wiosettings{'MAILSERVER'};
	my $smtpport = $wiosettings{'MAILSMTP'};
	my $from_address = $wiosettings{'MAILFROM'};
	my $username = $wiosettings{'MAILNAME'};
	my $password = $wiosettings{'MAILPW'};
	my $to_address1 = $wiosettings{'MAILTO1'};
	my $to_address2 = $wiosettings{'MAILTO2'};
	my $to_address3 = $wiosettings{'MAILTO3'};
	my $sub = $Lang::tr{'wio_mail_sub'};
	my $msg = $Lang::tr{'wio_mail_msg'};

	my $mailcall = "$mailclient -q -l $logfile -f $from_address -t $to_address1 -t $to_address2 -t $to_address3 -u $sub -m $msg -s $smtpserver:$smtpport";

	if ( $username eq '' && $password eq '' ) { system("$mailcall"); }
	elsif ( $password eq '' ) { system("$mailcall -xu $username"); }
	else { system("$mailcall -xu $username -xp '$password'"); }
}

############################################################################################################################
# sendemail log function
############################################################################################################################

if ( $wiosettings{'ACTION'} eq $Lang::tr{'wio_mail_log'} )
{

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio_mail_log_events'}, 1, '');
&Header::openbigbox('100%', 'left');

my $text = `/usr/bin/tail -n 25 $logfile`;
my $output = &Header::cleanhtml($text,"y");

&Header::openbox('100%', 'left', $Lang::tr{'wio_mail_log_events'});
	print "<table width='100%'>";
	print "<tr><td><pre>$output</pre></td></tr>";
	print "</table>";
&Header::closebox();

print <<END;
<table width='50%' align='center'>
<tr>
	<td align='center' width='25%'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='SUBMIT' name='ACTION' value='$Lang::tr{'wio_back'}'></form></td>
</tr>
</table>
END

&Header::closebigbox();
&Header::closepage();

exit 0;
}

############################################################################################################################
# sort function
############################################################################################################################

if ($ENV{'QUERY_STRING'} =~ /$sortstring/ )
{
	&General::readhash("$wiosettings", \%wiosettings);

	my $newsort = $ENV{'QUERY_STRING'};
	my $actual = $wiosettings{'SORT'};

	if ($actual =~ $newsort)
	{
		my $Rev = '';

		if ($actual !~ 'Rev')
		{
			$Rev = 'Rev';
		}

		$newsort.=$Rev;
	}

	system("/bin/sed -i 's#$wiosettings{'SORT'}#$newsort#g' $wiosettings");
	chmod ( 0644, $ipadrfile ); 
	chown ( $owner, $group, $ipadrfile );
	
	$wiosettings{'SORT'} = $newsort;
	map (delete ($wiosettings{$_}) ,(@nosaved,'ACTION','ID'));

	&SortDataFile;
	$wiosettings{'ACTION'} = 'SORT';
	map ($wiosettings{$_} = '' ,@nosaved,'ID');
}

############################################################################################################################
# main part
############################################################################################################################

ERROR:

&General::readhash("$wiosettings", \%wiosettings);

$selected{'CRON'}{'5'} = '';
$selected{'CRON'}{'10'} = '';
$selected{'CRON'}{'15'} = '';
$selected{'CRON'}{'20'} = '';
$selected{'CRON'}{'25'} = '';
$selected{'CRON'}{'30'} = '';
$selected{'CRON'}{'35'} = '';
$selected{'CRON'}{'40'} = '';
$selected{'CRON'}{'45'} = '';
$selected{'CRON'}{'50'} = '';
$selected{'CRON'}{'55'} = '';
$selected{'CRON'}{$wiosettings{'CRON'}} = "selected='selected'";

$selected{'TIMEOUT'}{'1'} = '';
$selected{'TIMEOUT'}{'2'} = '';
$selected{'TIMEOUT'}{'3'} = '';
$selected{'TIMEOUT'}{'4'} = '';
$selected{'TIMEOUT'}{'5'} = '';
$selected{'TIMEOUT'}{'6'} = '';
$selected{'TIMEOUT'}{'7'} = '';
$selected{'TIMEOUT'}{'8'} = '';
$selected{'TIMEOUT'}{'9'} = '';
$selected{'TIMEOUT'}{'10'} = '';
$selected{'TIMEOUT'}{$wiosettings{'TIMEOUT'}} = "selected='selected'";

$checked{'ENABLE'}{'off'} = '';
$checked{'ENABLE'}{'on'} = '';
$checked{'ENABLE'}{$wiosettings{'ENABLE'}} = "checked='checked'";

$checked{'UPDATE'}{'off'} = '';
$checked{'UPDATE'}{'on'} = '';
$checked{'UPDATE'}{$wiosettings{'UPDATE'}} = "checked='checked'";

$checked{'LOGGING'}{'off'} = '';
$checked{'LOGGING'}{'on'} = '';
$checked{'LOGGING'}{$wiosettings{'LOGGING'}} = "checked='checked'";

$checked{'MAIL'}{'off'} = '';
$checked{'MAIL'}{'on'} = '';
$checked{'MAIL'}{$wiosettings{'MAIL'}} = "checked='checked'";

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'wio'}, 1, $refreshpage);
&Header::openbigbox('100%', 'LEFT', '');

INFO:

###############
# DEBUG DEBUG
#&Header::openbox('100%', 'left', 'DEBUG');
#my $debugCount = 0;
#foreach my $line (sort keys %wiosettings) {
#  print "$line = $wiosettings{$line}<br />\n";
#  $debugCount++;
#}
#print "&nbsp;Count: $debugCount\n";
#&Header::closebox();
# DEBUG DEBUG
###############

if ($errormessage) {
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#FF0000'>$errormessage</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

if ($arp_msg) {
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#FF0000'>$arp_msg</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

if ($backup_errormessage) {
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#FF0000'>$backup_errormessage</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

if ($import_errormessage) {
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_error'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#FF0000'>$import_errormessage</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

if ($infomessage) {
	&Header::openbox('100%', 'LEFT', $Lang::tr{'wio_info'});
		print "<table width='100%'>";
		print "<tr><td><font class='base' color='#0000FF'>$infomessage&nbsp;</font></td></tr>";
		print "</table>";
	&Header::closebox();
}

if ($wiosettings{'UPDATE'} eq 'on') {

	my $addoncheck = &wiofunc::updatecheck($addonversion);

	if (($addonversion lt $addoncheck) && (-e $addonupdate))
	{
		unlink($addonupdate);
	}

if ( ! -e "$addonupdate" )
{
	&Header::openbox('100%', 'left', $Lang::tr{'wio_update_msg'});
		print "<table width='100%'>";
		print "<tr><td><font color='#0000FF'>$Lang::tr{'wio_update_info'}</font><a href='http://www.ipcop.h-loit.de' target='_blank'><font color='#0000FF'> http://www.ipcop.h-loit.de!</font></a></td></tr>";
		print "</table>";
	&Header::closebox();
}
}

############################################################################################################################
# wio configuration
############################################################################################################################

if ( $wiosettings{'ENABLE'} eq 'on' && $wiosettings{'ID'} eq '' && $errormessage eq '' )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio settings'});

print <<END;
<table width='100%'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_settings_msg'}</b></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='28%' align='right'>$Lang::tr{'wio_edit'}</td>
	<td width='16%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='hidden' name='ACTION' value='$Lang::tr{'edit'}1' /><input type='submit' name='SUBMIT' value='$Lang::tr{'edit'}' /></form></td>
END

print "<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>\n";

if (($wiosettings{'MAIL'} eq 'on' && $wiosettings{'MAILFROM'} ne '' && $wiosettings{'MAILSERVER'} ne '' && $wiosettings{'MAILSMTP'} ne '') && ($wiosettings{'MAILTO1'} ne '' || $wiosettings{'MAILTO2'} ne '' || $wiosettings{'MAILTO3'} ne ''))
{
print <<END;
	<td width='5%'>&nbsp;</td>
	<td width='17%' align='center'><input type='submit' name='ACTION' value='$Lang::tr{'wio_mail_test'}' /></td>
	<td width='17%' align='center'><input type='submit' name='ACTION' value='$Lang::tr{'wio_mail_log'}' /></td>
	<td width='17%' align='center'><input type='submit' name='ACTION' value='$Lang::tr{'wio_mail_log_clear'}' /></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END
}
else
{
print <<END;
	<td width='5%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END
}

print <<END;
<hr>
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='16%'>&nbsp;</td>
	<td width='5%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_backup_msg'}</b></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='28%' align='right'>$Lang::tr{'wio_backup'}</td>
	<td width='16%' align='center'><input type='submit' name='ACTION' value='$Lang::tr{'wio_backup_create'}' /></td>
	<td width='5%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
	<td width='17%'>&nbsp;</td>
</tr>
<tr>
	<td colspan='6' align='right' valign='bottom'><small><a href="http://www.ipcop.h-loit.de" target="_blank"><font color='${Header::table1colour}'><b>$addonname Ver. $addonversion</b></font></a></small></td>
</tr>
</table>
END

print "</form>";

&Header::closebox();
}

if ( $wiosettings{'ENABLE'} eq 'off' )
{
&Header::openbox('100%', 'left', $Lang::tr{'wio settings'});

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td width='30%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_settings_msg'}</b></td>
	<td width='22%'>&nbsp;</td>
	<td width='18%'>&nbsp;</td>
	<td width='30%'>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>$Lang::tr{'wio_settings_msg_hint'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio enabled'}</td>
	<td align='left'><input type='checkbox' name='ENABLE' $checked{'ENABLE'}{'on'} /></td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio cron'}</td>
	<td align='left'><select size='1' name='CRON' size='5'>
		<option $selected{'CRON'}{'5'}>5</option>
		<option $selected{'CRON'}{'10'}>10</option>
		<option $selected{'CRON'}{'15'}>15</option>
		<option $selected{'CRON'}{'20'}>20</option>
		<option $selected{'CRON'}{'25'}>25</option>
		<option $selected{'CRON'}{'30'}>30</option>
		<option $selected{'CRON'}{'35'}>35</option>
		<option $selected{'CRON'}{'40'}>40</option>
		<option $selected{'CRON'}{'45'}>45</option>
		<option $selected{'CRON'}{'50'}>50</option>
		<option $selected{'CRON'}{'55'}>55</option>
		</select>&nbsp;$Lang::tr{'wio min'}</td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio timeout'}</td>
	<td align='left'><select size='1' name='TIMEOUT' size='5'>
		<option $selected{'TIMEOUT'}{'1'}>1</option>
		<option $selected{'TIMEOUT'}{'2'}>2</option>
		<option $selected{'TIMEOUT'}{'3'}>3</option>
		<option $selected{'TIMEOUT'}{'4'}>4</option>
		<option $selected{'TIMEOUT'}{'5'}>5</option>
		<option $selected{'TIMEOUT'}{'6'}>6</option>
		<option $selected{'TIMEOUT'}{'7'}>7</option>
		<option $selected{'TIMEOUT'}{'8'}>8</option>
		<option $selected{'TIMEOUT'}{'9'}>9</option>
		<option $selected{'TIMEOUT'}{'10'}>10</option>
		</select>&nbsp;$Lang::tr{'wio sec'}</td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_update'}</td>
	<td align='left'><input type='checkbox' name='UPDATE' $checked{'UPDATE'}{'on'} /></td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_logging'}</td>
	<td align='left'><input type='checkbox' name='LOGGING' $checked{'LOGGING'}{'on'} /></td>
	<td align='left'>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mail_enabled'}</td>
	<td align='left'><input type='checkbox' name='MAIL' $checked{'MAIL'}{'on'}></td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailsender'}:</td>
	<td align='left'><input type='text' name='MAILFROM' value='$wiosettings{'MAILFROM'}' size='30' tabindex='1'></td>
	<td align='right'>$Lang::tr{'wio_mailserver'}:</td>
	<td align='left'><input type='text' name='MAILSERVER' value='$wiosettings{'MAILSERVER'}' size='30' tabindex='5'></td>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto1'}:</td>
	<td align='left'><input type='text' name='MAILTO1' value='$wiosettings{'MAILTO1'}' size='30' tabindex='2'></td>
	<td align='right'>$Lang::tr{'wio_mailsmtpport'}:</td>
	<td align='left'><input type='text' name='MAILSMTP' value='$wiosettings{'MAILSMTP'}' size='2' tabindex='6'></td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto2'}:</td>
	<td align='left'><input type='text' name='MAILTO2' value='$wiosettings{'MAILTO2'}' size='30' tabindex='3'></td>
	<td align='right'>$Lang::tr{'wio_mailserveruser'}:</td>
	<td align='left'><input type='text' name='MAILNAME' value='$wiosettings{'MAILNAME'}' size='30' tabindex='7'></td>
</tr>
<tr>
	<td align='right'>$Lang::tr{'wio_mailto3'}:</td>
	<td align='left'><input type='text' name='MAILTO3' value='$wiosettings{'MAILTO3'}' size='30' tabindex='4'></td>
	<td align='right'>$Lang::tr{'wio_mailserverpw'}:</td>
	<td align='left'><input type='password' name='MAILPW' value='$wiosettings{'MAILPW'}' size='30' tabindex='8'></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td align='left'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_save'}1' /><input type='submit' name='submit' value='$Lang::tr{'wio_save'}' /></td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
</tr>
</table>
</form>
<hr>
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<table width='100%'>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='30%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_backup_import_msg'}</b></td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='30%'>$Lang::tr{'wio_backup_import_hint'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='30%' align='right'>$Lang::tr{'wio_backup_import'}</td>
	<td width='35%' align='left'><input type='file' name='BACKUPFILE' size='30' /></td>
	<td width='14%'>&nbsp;</td>
	<td width='21%'>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='35%' align='left'><input type='SUBMIT' name='ACTION' value='$Lang::tr{'wio_backup_fileimport'}' /></td>
	<td width='14%'>&nbsp;</td>
	<td width='23%'>&nbsp;</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	<td align='right'><small><a href="http://www.ipcop.h-loit.de" target="_blank"><font color='${Header::table1colour}'><b>$addonname Ver. $addonversion</b></font></a></small></td>
</tr>
</table>
</form>
END

&Header::closebox();
}

############################################################################################################################
# wio client status
############################################################################################################################

if ( $wiosettings{'ENABLE'} eq 'on')
{
	my $sortarrow1 = '';
	my $sortarrow2 = '';
	my $sortarrow3 = '';

	if ($wiosettings{'SORT'} eq 'IPADRRev')
	{
		$sortarrow1 = $Header::sortdn;
	}
	elsif ($wiosettings{'SORT'} eq 'IPADR')
	{
		$sortarrow1 = $Header::sortup;
	}
	elsif ($wiosettings{'SORT'} eq 'HOSTRev')
	{
		$sortarrow2 = $Header::sortdn;
	}
	elsif ($wiosettings{'SORT'} eq 'HOST')
	{
		$sortarrow2 = $Header::sortup;
	}
	elsif ($wiosettings{'SORT'} eq 'REMARKRev')
	{
		$sortarrow3 = $Header::sortdn;
	}	
	else
	{
		$sortarrow3 = $Header::sortup;
	}

	if ( $wiosettings{'ID'} eq '' && $errormessage eq '' )
	{
		&Header::openbox('100%', 'left', $Lang::tr{'wio stat'});

		my $nr = 0;

		open(FILE ,"<$ipadrfile");
		flock FILE, 2;
		my @addressfile = <FILE>;
		close(FILE);

		foreach my $line (@addressfile)
		{
			chomp($line);

			my @temp = split (/\,/, $line);

			$ipadr = $temp[0];
			$host = $temp[1];
			$on = $temp[2];
			$remark = $temp[3];
			$dyndns = $temp[4];
			$sendemailon = $temp[5];
			$sendemailoff = $temp[6];
			$pingmethode = $temp[7];
			$online = $temp[8];

			my $file_time = ();

			if ( $on ne 'on' ) { next; }

			if ( -e "$redactive" && ( defined($dyndns) && ( $dyndns =~ 'on' ))) {

				my @fetchdyndnsip = ();
				@fetchdyndnsip = gethostbyname($host);

				if ( $fetchdyndnsip[0] ne '' )
				{
					@fetchdyndnsip = map ( &Socket::inet_ntoa($_), @fetchdyndnsip[4..$#fetchdyndnsip]);

					if ($fetchdyndnsip[0] ne $temp[0])
					{
						$ipadr = $fetchdyndnsip[0];
						system("/bin/sed -i 's#$temp[0]#$fetchdyndnsip[0]#g' $ipadrfile");
						chmod ( 0644, $ipadrfile ); 
						chown ( $owner, $group, $ipadrfile );
					}
				}
			}

			if ( $pingmethode eq 'ip' )
			{
				if ( defined($dyndns) && ( $dyndns eq 'on' ) )
				{
					push(@tmpfilename,("updown-$host.png"));
				}
				else
				{
					push(@tmpfilename,("updown-$ipadr.png"));
				}

				if ( -e "$imagedir/$tmpfilename[$nr]" )
				{
					my $file = "$imagedir/$tmpfilename[$nr]";
					my @Info = stat($file);
					my $time = "$Info[9]\n";
					$file_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
				}
				else
				{
					$file_time = ();
				}
			}
			else
			{
				push (@tmpfilename,("updown-$host.png"));

				if ( -e "$imagedir/$tmpfilename[$nr]" )
				{
					my $file = "$imagedir/$tmpfilename[$nr]";
					my @Info = stat($file);
					my $time = "$Info[9]\n";
					$file_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
				}
				else
				{
					$file_time = ();
				}
			}

			if ($file_time)
			{
				push (@dates,($file_time));
				push (@ipaddresses,($ipadr));
				push (@names,($host));
				push (@remark,($remark));
				my $filnavn = $tmpfilename[$nr];
				push (@filenames,($filnavn));
				$nr++;
			}
			else
			{
				$file_time = "-";
				push (@dates,($file_time));
				push (@ipaddresses,($ipadr));
				push (@names,($host));
				push (@remark,($remark));
				my $filnavn = "wionoimage.png";
				push (@filenames,($filnavn));
				$nr++;
			}
		}

############################################################################################################################
# wan connection
############################################################################################################################

my $wan_time = ();
my $timestr = ();

if ( -e "$redactive" )
{
	my $file = "$redactive";
	my @Info = stat($file);
	my $time = "$Info[9]\n";
	$wan_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
	$timestr = &General::age("${General::swroot}/red/active");
}
else
{
	$wan_time = '-';
	$timestr = '-';
}

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_wan_con'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>

<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio_wancheck'}</b></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio ipadress'}</b></a></td>
	<td width='7%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio network'}</b></td>
	<td width='19%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio_lanname'}</b></a></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio_wanname'}</b></td>
	<td width='12%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio image'}</b></td>
	<td width='8%' class='boldbase' align='center' height='20'><b>$Lang::tr{'action'}</b></td>
</tr>
<tr>
	<td align='center' height='20'><font color='${Header::colourred}'><b>$timestr</b></font></td>
	<td align='center' height='20'><font color='${Header::colourred}'><b>$redip</b></font></td>
	<td align='center' height='20'><img align='absmiddle' src='$imgstatic/wiored.png' alt='WAN' title='WAN' /></td>
	<td align='center' height='20'><font color='${Header::colourgreen}'><b>$fqdn</font></b></td>
	<td align='center' height='20'><font color='${Header::colourred}'><b>$redhostname</b></font></td>
	<td align='center' height='20'><img align='absmiddle' src='/graphs/wio/wiowan.png' /></td>
END

if ( -e "$redactive" )
{
print <<END;
	<td align='center' height='20'><form method='post' action='/cgi-bin/dial.cgi' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'hangup'}' /></form></td>
</tr>
</table>
END
}
else
{
print <<END;
	<td align='center' height='20'><form method='post' action='/cgi-bin/dial.cgi' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'dial'}' /></form></td>
</tr>
</table>
END
}

print <<END;
<table width='100%'>
<tr>
	<td colspan='2' height='35'><hr></td>
</tr>
</table>
END

############################################################################################################################
# dyndns hostname(s)
############################################################################################################################

if ( -s "$dyndnsfile" )
{

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_dyndnshostname'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='18%' align='center' height='20'><b>$Lang::tr{'wio_dyndns_hosts'}</b></td>
	<td>&nbsp;</tr>
</tr>
<tr>
	<td align='left' colspan='2' height='20'>
END

my $ipadr = &General::GetDyndnsRedIP;
my $key = 0;
my $zahl = ();
$zahl = @ddns;

foreach my $line (@ddns)
{
	chomp($line);
	my @temp = split (/\,/, $line);

	my $sync = "<font color='${Header::colourblue}'>";

	if ($temp[7] eq "on")
	{
		$sync = (&General::DyndnsServiceSync ($ipadr,$temp[1],$temp[2]) ? "<font color='${Header::colourgreen}'>": "<font color='${Header::colourred}'>") ;
	}

	print "&nbsp;$sync<b>$temp[1].$temp[2]</b></font>";

	if ( $key ne ($zahl-1) )
	{
		print "<b> |</b>";
	}

$key++;
}

print <<END;
	</td>
</tr>
</table>
<table width='100%'>
<tr>
	<td colspan='2' height='35'><hr></td>
</tr>
</table>
END
}

############################################################################################################################
# vpn connection(s)
############################################################################################################################

if ( -e "$vpnpid" )
{

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_vpn_con'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio checked'}</b></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'name'}</b></td>
	<td width='7%' class='boldbase' align='center' height='20'><b>$Lang::tr{'type'}</b></td>
	<td width='19%' class='boldbase' align='center' height='20'><b>$Lang::tr{'common name'}</b></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'remark'}</b></td>
	<td width='12%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio image'}</b></td>
	<td width='8%' class='boldbase' align='center' height='20'><b></b></td>
</tr>
END

my $idvpn = 0;
my $vpn_time = '';

if ( -e "/proc/net/ipsec_eroute" )
{
	my $file = "/proc/net/ipsec_eroute";
	my @Info = stat($file);
	my $time = "$Info[9]\n";
	$vpn_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
}
else
{
	$vpn_time = '-';
}

foreach my $key (keys %vpnconfighash)
{
	if ($idvpn % 2)
	{
		print "<tr bgcolor='${Header::table1colour}'>";
	}
	else
	{
		print "<tr>"; 
	}

	print "<td align='center' height='22'>$vpn_time</td>";
	print "<td align='center' height='22'>$vpnconfighash{$key}[1]</td>";

	if ( $vpnconfighash{$key}[3] eq 'host' )
	{
		print "<td align='center' height='22'><img align='absmiddle' src='$imgstatic/wiorw.png' alt='$Lang::tr{'wio_rw'}' title='$Lang::tr{'wio_rw'}' /></td>";
	}
	elsif ( $vpnconfighash{$key}[3] eq 'net' )
	{
		print "<td align='center' height='22'><img align='absmiddle' src='$imgstatic/wion2n.png' alt='$Lang::tr{'wio_n2n'}' title='$Lang::tr{'wio_n2n'}' /></td>";
	}

	if ($vpnconfighash{$key}[2] eq '%auth-dn')
	{
		print "<td align='center' height='22'>$vpnconfighash{$key}[9]</td>";
	}
	elsif ($vpnconfighash{$key}[4] eq 'cert')
	{
		print "<td align='center' height='22'>$vpnconfighash{$key}[2]</td>";
	}
	else
	{
		print "<td align='center' height='22'>-</td>";
	}

	if ($vpnconfighash{$key}[25] ne '')
	{
		print "<td align='center' height='22'>$vpnconfighash{$key}[25]</td>";
	}
	else
	{
		print "<td align='center' height='22'>-</td>";
	}

	my $active = "<img align='absmiddle' src='/graphs/wio/wiovpnclosed.png' />";

	if ($vpnconfighash{$key}[0] eq 'off')
	{
		if ( ! -e "$imagedir/wiovpnoff.png" )
		{
			&wiofunc::vpnoff();
		}

		$active = "<img align='absmiddle' src='/graphs/wio/wiovpnoff.png' />";
	}
	else
	{
		foreach my $line (@vpnstatus)
		{
			if ($line =~ /\"$vpnconfighash{$key}[1]\".*IPsec SA established/)
			{
				if ( ! -e "$imagedir/wiovpnopen.png" )
				{
					&wiofunc::vpnopen();
				}

				$active = "<img align='absmiddle' src='/graphs/wio/wiovpnopen.png' />";
			}
		}
	}

print <<END;
<td align='center' height='22'>$active</td>
<td align='center' height='22'></td>
</tr>
END

$idvpn++;
}

print "</table>\n";

print <<END;
<table width='100%'>
<tr>
	<td height='35'><hr></td>
</tr>
</table>
END

}
else
{
}

############################################################################################################################
# openvpn connection(s)
############################################################################################################################

if ( -e "$ovpnpid" )
{

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_ovpn_con'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio checked'}</b></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'name'}</b></td>
	<td width='7%' class='boldbase' align='center' height='20'><b>$Lang::tr{'type'}</b></td>
	<td width='19%' class='boldbase' align='center' height='20'><b>$Lang::tr{'common name'}</b></td>
	<td width='18%' class='boldbase' align='center' height='20'><b>$Lang::tr{'remark'}</b></td>
	<td width='12%' class='boldbase' align='center' height='20'><b>$Lang::tr{'wio image'}</b></td>
	<td width='8%' class='boldbase' align='center' height='20'><b></b></td>
</tr>
END

my $idovpn = 0;
my $vpn_time = '';
my $cn = '';
my @match = '';

if ( -e "/var/log/ovpnserver.log" )
{
	my $file = "/var/log/ovpnserver.log";
	my @Info = stat($file);
	my $time = "$Info[9]\n";
	$vpn_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
}
elsif ( -e "/var/ipcop/ovpn/server.log" )
{
	my $file = "/var/ipcop/ovpn/server.log";
	my @Info = stat($file);
	my $time = "$Info[9]\n";
	$vpn_time = strftime("%d.%m.%Y - %H:%M:%S",localtime($time));
}
else
{
	$vpn_time = '-';
}

foreach my $key (keys %ovpnconfighash)
{
	if ($idovpn % 2)
	{
		print "<tr bgcolor='${Header::table1colour}'>";
	}
	else
	{
		print "<tr>"; 
	}

	print "<td align='center' height='22'>$vpn_time</td>";
	print "<td align='center' height='22'>$ovpnconfighash{$key}[1]</td>";

	if ( $ovpnconfighash{$key}[3] eq 'host' )
	{
		print "<td align='center' height='22'><img align='absmiddle' src='$imgstatic/wiorw.png' alt='$Lang::tr{'wio_rw'}' title='$Lang::tr{'wio_rw'}' /></td>";
	}
	elsif ( $ovpnconfighash{$key}[3] eq 'net' )
	{
		print "<td align='center' height='22'><img align='absmiddle' src='$imgstatic/wion2n.png' alt='$Lang::tr{'wio_n2n'}' title='$Lang::tr{'wio_n2n'}' /></td>";
	}

	if ($ovpnconfighash{$key}[2] eq '%auth-dn')
	{
		print "<td align='center' height='22'>$ovpnconfighash{$key}[9]</td>";
	}
	elsif ($ovpnconfighash{$key}[4] eq 'cert')
	{
		print "<td align='center' height='22'>$ovpnconfighash{$key}[2]</td>";
	}
	else {
		print "<td align='center' height='22'>-</td>";
	}

	if ($ovpnconfighash{$key}[25] ne '')
	{
		print "<td align='center' height='22'>$ovpnconfighash{$key}[25]</td>";
	}
	else {
		print "<td align='center' height='22'>-</td>";
	}

	my $active = "<img align='absmiddle' src='/graphs/wio/wioovpnclosed.png' />";

	if ($ovpnconfighash{$key}[0] eq 'off')
	{
		if ( ! -e "$imagedir/wioovpnoff.png" )
		{
			&wiofunc::ovpnoff();
		}

		$active = "<img align='absmiddle' src='/graphs/wio/wioovpnoff.png' />";
	}
	else
	{
		if ( $ovpnconfighash{$key}[3] eq 'host' )
		{
			foreach my $line (@ovpnstatus)
			{
				chomp($line);
				
				if ( $line =~ /^(.+),(\d+\.\d+\.\d+\.\d+\:\d+),(\d+),(\d+),(.+)/)
				{
					@match = split (m/^(.+),(\d+\.\d+\.\d+\.\d+\:\d+),(\d+),(\d+),(.+)/, $line);

				if ( $match[1] ne "Common Name" )
				{
					$cn = $match[1];
				}

				$cn =~ s/[_]/ /g;
				}

				if ( defined($cn) && ( $cn eq $ovpnconfighash{$key}[2] ) )
				{
					if ( ! -e "$imagedir/wioovpnopen.png" )
					{
						&wiofunc::ovpnopen();
					}

					$active = "<img align='absmiddle' src='/graphs/wio/wioovpnopen.png' />";
				}
			}
		}
		else
		{
			my @ovpnsubnet = split ("\/", $ovpnconfighash{$key}[13]);
			my @ovpnip = split (/\./, $ovpnsubnet[0]);
			my $pingip = "";
				
			if ( $ovpnconfighash{$key}[6] eq 'server' )
			{
				$pingip = "$ovpnip[0].$ovpnip[1].$ovpnip[2].2";
			}
			else
			{
				$pingip = "$ovpnip[0].$ovpnip[1].$ovpnip[2].1";
			}

			my $p = Net::Ping->new("udp",1);

			if ( $p->ping($pingip) )
			{
				if ( ! -e "$imagedir/wioovpnopen.png" )
				{
					&wiofunc::ovpnopen();
				}

				$active = "<img align='absmiddle' src='/graphs/wio/wioovpnopen.png' />";

			}
			$p->close();				
		}
	}

print <<END;
<td align='center' height='22'>$active</td>
<td align='center' height='22'></td>
</tr>
END

$idovpn++;
}

print "</table>\n";

print <<END;
<table width='100%'>
<tr>
	<td height='35'><hr></td>
</tr>
</table>
END

}
else
{
}

############################################################################################################################
# arp table entries
############################################################################################################################

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'arp table entries'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='18%' align='center' nowrap height='20'><b>$Lang::tr{'wio_hwaddress'}</b></td>
	<td width='18%' align='center' nowrap height='20'><b>$Lang::tr{'wio ipadress'}</b></td>
	<td width='7%'  align='center' nowrap height='20'><b>$Lang::tr{'wio network'}</b></td>
	<td width='19%' align='center' nowrap height='20'><b>$Lang::tr{'wio name'}</b></td>
	<td width='18%' align='center' nowrap height='20'><b>$Lang::tr{'remark'}</b></td>
	<td width='12%' align='center' nowrap height='20'><b>$Lang::tr{'wio_iface'}</b></td>
	<td width='4%' align='center' nowrap height='20'><b>$Lang::tr{'wio_hwtype'}</b></td>
	<td width='4%' align='center' nowrap height='20'><b>$Lang::tr{'action'}</b></td>
</tr>
END

my $idarp = 0;

my @arp = `/sbin/arp -a`;

foreach my $line (sort @arp)
{
	chomp($line);

	my @line = split (/ /, $line);
	my @arpip = split (/\(/, $line[1]);
	my @arpipnew = split (/\)/, $arpip[1]);
	my @hwtype = split (/\[/, $line[4]);
	my @hwtypenew = split (/\]/, $hwtype[1]);

	push( @arpcache, "$arpipnew[0],$line[0]\n" );

	if ($idarp % 2)
	{
		print "<tr bgcolor='${Header::table1colour}'>";
	}
	else
	{
		print "<tr>"; 
	}

	if ( $line[4] eq 'on' )
	{
		print "<td width='18%' align='center' height='20' nowrap>&nbsp;</td>";
	}
	else
	{
		print "<td width='18%' align='center' height='20' nowrap>$line[3]</td>";
	}
	print "<td width='18%' align='center' height='20' nowrap>$arpipnew[0]</td>";

	my @devs_color = ('GREEN_DEV','BLUE_DEV','ORANGE_DEV','RED_DEV','GRAY1_DEV','GRAY2_DEV','GRAY3_DEV','GRAY4_DEV');
	my @devs_img = ('wiogreen.png','wioblue.png','wioorange.png','wiored.png','wiogray.png','wiogray.png','wiogray.png','wiogray.png');
	my @devs_alt = ('lan','wireless','dmz','internet','gray','gray','gray','gray');
	my @devs_net = ('eth0','eth1','eth2','eth3','eth4','eth5','eth6','eth7','ath0');
	my $in = 0;
	
	foreach my $netcolor (@devs_color)
	{
		foreach my $netinterface (@devs_net)
		{
			if (($netsettings{$netcolor} eq $netinterface) && ( $line[5] eq $netinterface || $line[6] eq $netinterface || $line[7] eq $netinterface ))
			{
				print "<td width='7%' align='center' height='20' nowrap><img src='$imgstatic/$devs_img[$in]' alt='$Lang::tr{$devs_alt[$in]}' title='$Lang::tr{$devs_alt[$in]}' /></td>";
			}
		}
		$in++;
	}

	if ( $line[0] eq '?' )
	{
		print "<td width='19%' align='center' height='20' nowrap>&nbsp;</td>";
	}
	else
	{
		print "<td width='19%' align='center' height='20' nowrap>$line[0]</td>";
	}

	if ( $line[5] eq 'eth0' || $line[5] eq 'eth1' || $line[5] eq 'eth2' || $line[5] eq 'eth3' || $line[5] eq 'ath0' )
	{
		print "<td width='18%' align='center' height='20' nowrap>$Lang::tr{'wio_incom'}</td>";
	}
	else
	{
		print "<td width='18%' align='center' height='20' nowrap>$Lang::tr{'wio_com'}</td>";
	}

	if ( $line[4] eq 'on' )
	{
		print "<td width='12%' align='center' height='20' nowrap>$line[5]</td>";
	}
	elsif ( $line[5] eq 'on' )
	{
		print "<td width='12%' align='center' height='20' nowrap>$line[6]</td>";
	}
	elsif ( $line[6] eq 'on' )
	{
		print "<td width='12%' align='center' height='20' nowrap>$line[7]</td>";
	}

	if ( $line[4] eq 'on' )
	{
		print "<td width='4%' align='center' height='20' nowrap>&nbsp;</td>";
	}
	else
	{
		print "<td width='4%' align='center' height='20' nowrap>$hwtypenew[0]</td>";
	}

	if ( $line[4] ne 'on' )
	{
		if ( "@ipaddresses" !~ /$arpipnew[0]/ )
		{
			print "<td width='4%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
			print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_add'}' />";
			print "<input type='image' name='$Lang::tr{'wio_add'}' src='/images/add.gif' align='absmiddle' alt='$Lang::tr{'wio_add'}' title='$Lang::tr{'wio_add'}' />";
			print "<input type='hidden' name='ID' value='$idarp' /></form></td>";
		}
		else
		{
			print "<td width='4%' align='center' height='20' nowrap><img src='$imgstatic/add_host.png' align='absmiddle' alt='$Lang::tr{'wio_no_add'}' title='$Lang::tr{'wio_no_add'}' /></td>";
		}
	}
	else
	{
		print "<td width='4%' align='center' height='20' nowrap>&nbsp;</td>";
	}

print <<END;
</tr>
END

$idarp++;
}

print "</table>\n";

print <<END;
<table width='100%'>
<tr>
	<td height='35'><hr></td>
</tr>
</table>
END

open( FILE, "> /var/log/wio/.arpcache" );
flock FILE, 2;
print FILE @arpcache;
close(FILE);

############################################################################################################################
# client status
############################################################################################################################

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_clients'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='3%' align='center' nowrap height='20'><b>$Lang::tr{'wio_id'}</b></td>
	<td width='15%' align='center' nowrap height='20'><b>$Lang::tr{'wio checked'}</b></td>
	<td width='18%' align='center' nowrap height='20'><a href='$ENV{'SCRIPT_NAME'}?IPADR'><b>$Lang::tr{'wio ipadress'}</b> $sortarrow1</a></td>
	<td width='7%' align='center' nowrap height='20'><b>$Lang::tr{'wio network'}</b></td>
	<td width='19%' align='center' nowrap height='20'><a href='$ENV{'SCRIPT_NAME'}?HOST'><b>$Lang::tr{'wio name'}</b> $sortarrow2</a></td>
	<td width='18%' align='center' nowrap height='20'><a href='$ENV{'SCRIPT_NAME'}?REMARK'><b>$Lang::tr{'remark'}</b> $sortarrow3</a></td>
	<td width='10%' align='center' nowrap height='20'><b>$Lang::tr{'wio image'}</b></td>
	<td width='2%' align='center' nowrap height='20'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_refresh'}' /><input type='image' name='$Lang::tr{'wio_refresh'}' src='/images/reload.gif' align='absmiddle' alt='$Lang::tr{'wio_refresh'}' title='$Lang::tr{'wio_refresh'}' /></form></td>
	<td width='8%' colspan='2' align='center' nowrap height='20'><b>$Lang::tr{'action'}</b></td>
END

my $idclient = 0;

for(my $a=0; $a<$nr; $a++)
{

if ($idclient % 2)
{
	print "<tr bgcolor='${Header::table1colour}'>";
}
else
{
	print "<tr>"; 
}

printf ("<td width='3%' align='center' nowrap> %02d</td>", $idclient+1);

print <<END;
	<td width='15%' align='center' nowrap>$dates[$a]</td>
	<td width='18%' align='center' nowrap>$ipaddresses[$a]</td>
END

$dotip = length($ipaddresses[$a]) - rindex($ipaddresses[$a],'.');

	SWITCH: {

		if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'GREEN_ADDRESS'}, $netsettings{'GREEN_NETMASK'}) ) 
		{
			print "<td width='7%' align='center' nowrap><img src='$imgstatic/wiogreen.png' alt='$Lang::tr{'lan'}' title='$Lang::tr{'lan'}' /></td>"; 
			last SWITCH;
		} 
		if ( -e "$vpnpid" ) {
		
		( $vpnipsec[0] ) and do	{

			foreach my $line (@vpnconfig)
			{
				my @temp = split (/\,/, $line);

				if ( $temp[4] eq 'net' )
				{
					my @temp0 = split ("/", $temp[12]);

					$vpnn2nip = $temp0[0];
					$vpnn2nmask = length($temp0[1]) - rindex($temp0[1],'.');

					if (substr($ipaddresses[$a],0,length($ipaddresses[$a])-$dotip) eq substr($vpnn2nip,0,length($vpnn2nip)-$vpnn2nmask))
					{
						print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiovpn.png' alt='$Lang::tr{'vpn'}' title='$Lang::tr{'vpn'}' /></td>";
						last SWITCH;
					}
				}
			}
		};

		( $vpnipsec[0] ) and do {

			foreach my $line (@vpnipsec)
			{
				my @temp = split (/[\t ]+/, $line);
				my @temp1 = split (/[\/:]+/, $temp[3]);

				$vpnrwip = $temp1[0];
				$vpnrwmask = length($temp1[0]) - rindex($temp1[0],'.');

				if ( $vpnrwip eq $ipaddresses[$a] )
				{
					print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiovpn.png' alt='$Lang::tr{'vpn'}' title='$Lang::tr{'vpn'}' /></td>";
					last SWITCH;
				}
			}
		};
		};

		if ( -e "$ovpnpid" ) {
		
		( $ovpnpid ) and do {

			my @matchovpn = ();
			my @realip = ();
			my $virtualip;

			foreach my $line (@ovpnstatus)
			{
				chomp($line);

				if ( $line =~ /^(\d+\.\d+\.\d+\.\d+),(.+),(\d+\.\d+\.\d+\.\d+\:\d+)/)
				{
					@matchovpn = split (m/^(\d+\.\d+\.\d+\.\d+),(.+),(\d+\.\d+\.\d+\.\d+\:\d+)/, $line);
					$virtualip = $matchovpn[1];
					@realip = split (/\:/, $matchovpn[3]);
				}
			}

			my @ovpnrw = split ("\/", $ovpnsettings{'DOVPN_SUBNET'});

			$ovpnrwip = $virtualip;
			$ovpnrwmask = $ovpnrw[1];

			if ( defined($realip[0]) && ( $realip[0] eq $ipaddresses[$a] ) )
			{
				print "<td width='7%' align='center' nowrap><img src='$imgstatic/wioovpn.png' alt='$Lang::tr{'ovpn'}' title='$Lang::tr{'ovpn'}' /></td>";
				last SWITCH;
			}

			if ( &General::IpInSubnet($ipaddresses[$a], $ovpnrwip, $ovpnrwmask) )
			{
				print "<td width='7%' align='center' nowrap><img src='$imgstatic/wioovpn.png' alt='$Lang::tr{'ovpn'}' title='$Lang::tr{'ovpn'}' /></td>"; 
				last SWITCH; 
			}							
			
			if ( &General::IpInSubnet($ipaddresses[$a], $ovpnrw[0], $ovpnrwmask) )
			{
				print "<td width='7%' align='center' nowrap><img src='$imgstatic/wioovpn.png' alt='$Lang::tr{'ovpn'}' title='$Lang::tr{'ovpn'}' /></td>"; 
				last SWITCH; 
			}							
		};

		( $ovpnpid ) and do { 

			foreach my $line (@ovpnconfig)
			{
				my @ovpnnet = split (/\,/, $line);

				if ( $ovpnnet[4] eq 'net' )
				{
					my @ovpnnet2net = split ("/", $ovpnnet[12]);

					$ovpnn2nip = $ovpnnet2net[0];
					$ovpnn2nmask = length($ovpnnet2net[1]) - rindex($ovpnnet2net[1],'.');

					if (substr($ipaddresses[$a],0,length($ipaddresses[$a])-$dotip) eq substr($ovpnn2nip,0,length($ovpnn2nip)-$ovpnn2nmask))
					{
						print "<td width='7%' align='center' nowrap><img src='$imgstatic/wioovpn.png' alt='$Lang::tr{'ovpn'}' title='$Lang::tr{'ovpn'}' /></td>"; 
						last SWITCH; 
					}
				}
			}
		};
		};

		( defined($netsettings{'BLUE_DEV'}) && defined($netsettings{'BLUE_ADDRESS'}) ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'BLUE_ADDRESS'}, $netsettings{'BLUE_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wioblue.png' alt='$Lang::tr{'wireless'}' title='$Lang::tr{'wireless'}' /></td>";
				last SWITCH;
			}
		};

		( defined($netsettings{'ORANGE_DEV'}) && defined($netsettings{'ORANGE_ADDRESS'}) ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'ORANGE_ADDRESS'}, $netsettings{'ORANGE_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wioorange.png' alt='$Lang::tr{'dmz'}' title='$Lang::tr{'dmz'}' /></td>";
				last SWITCH;
			}
		};

		( defined($netsettings{'GRAY1_DEV'}) && $netsettings{'GRAY1_DEV'} ne 'disabled' ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'GRAY1_ADDRESS'}, $netsettings{'GRAY1_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiogray.png' alt='$Lang::tr{'wio_gray'}' title='$Lang::tr{'wio_gray'}' /></td>";
				last SWITCH;
			}
		};

		( defined($netsettings{'GRAY2_DEV'}) && $netsettings{'GRAY2_DEV'} ne 'disabled' ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'GRAY2_ADDRESS'}, $netsettings{'GRAY2_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiogray.png' alt='$Lang::tr{'wio_gray'}' title='$Lang::tr{'wio_gray'}' /></td>";
				last SWITCH;
			}
		};

		( defined($netsettings{'GRAY3_DEV'}) && $netsettings{'GRAY3_DEV'} ne 'disabled' ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'GRAY3_ADDRESS'}, $netsettings{'GRAY3_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiogray.png' alt='$Lang::tr{'wio_gray'}' title='$Lang::tr{'wio_gray'}' /></td>";
				last SWITCH;
			}
		};

		( defined($netsettings{'GRAY4_DEV'}) && $netsettings{'GRAY4_DEV'} ne 'disabled' ) and do {

			if ( &General::IpInSubnet($ipaddresses[$a], $netsettings{'GRAY4_ADDRESS'}, $netsettings{'GRAY4_NETMASK'}) ) 
			{
				print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiogray.png' alt='$Lang::tr{'wio_gray'}' title='$Lang::tr{'wio_gray'}' /></td>";
				last SWITCH;
			}
		};

		print "<td width='7%' align='center'><img align='absmiddle' src='$imgstatic/wiored.png' alt='$Lang::tr{'internet'}' title='$Lang::tr{'internet'}' /></td>";
	}

print <<END;
	<td width='19%' align='center'>$names[$a]</td>
END

if ( defined($remark[$a]) && ( $remark[$a] ne '' ) )
{
	print "<td width='18%' align='center'>$remark[$a]</td>";
}
else
{
	print "<td width='18%' align='center'>&nbsp;</td>";
}

print <<END;
	<td width='12%' align='center' colspan='2'><img align='absmiddle' src='/graphs/wio/$filenames[$a]' /></td>
	<td width='4%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
	<input type='hidden' name='ACTION' value='$Lang::tr{'wio_onoff_refresh'}' />
	<input type='image' name='$Lang::tr{'wio_onoff_refresh'}' src='/images/reload.gif' align='absmiddle' alt='$Lang::tr{'wio_onoff_refresh'}' title='$Lang::tr{'wio_onoff_refresh'}' />
	<input type='hidden' name='ID' value='$idclient' /></form></td>

	<td width='4%' align = 'center'><form method= 'post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
	<input type='hidden' name='ACTION' value='$Lang::tr{'wio_graphs'}' />
	<input type='image' name='$Lang::tr{'wio_graphs'}' src='$imgstatic/wiograph.gif' align='absmiddle' alt='$Lang::tr{'wio_graphs'}' title='$Lang::tr{'wio_graphs'}' />
	<input type='hidden' name='ID' value='$idclient' /></form></td>
	</tr>
END

$idclient++;
}

print "</table>\n";

print <<END;
<table width='100%'>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

print <<END;
<table width='100%'>
<tr>
	<td colspan='2' height='10'></td>
</tr>
</table>
<table border='0' width='100%' align='left' cellspacing='0' cellpadding='0'>
<tr>
	<td width='15%' align='left' height='20'>&nbsp;<b>$Lang::tr{'legend'}:</b></td>
	<td width='7%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wiogreen.png' alt='$Lang::tr{'lan'}' title='$Lang::tr{'lan'}' />&nbsp;$Lang::tr{'lan'}</td>
	<td width='10%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wiored.png' alt='$Lang::tr{'internet'}' title='$Lang::tr{'internet'}' />&nbsp;$Lang::tr{'internet'}</td>
END

if ( defined($netsettings{'ORANGE_DEV'}) && defined($netsettings{'ORANGE_ADDRESS'}) )
{
print <<END;
     <td width='7%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wioorange.png' alt='$Lang::tr{'dmz'}' title='$Lang::tr{'dmz'}' />&nbsp;$Lang::tr{'dmz'}</td>
END
}
else
{
print <<END;
     <td width='7%' align='left' height='20'>&nbsp;</td>
END
}

if ( defined($netsettings{'BLUE_DEV'}) && defined($netsettings{'BLUE_ADDRESS'}) )
{
print <<END;
     <td width='10%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wioblue.png' alt='$Lang::tr{'wireless'}' title='$Lang::tr{'wireless'}' />&nbsp;$Lang::tr{'wireless'}</td>
END
}
else
{
print <<END;
     <td width='10%' align='left' height='20'>&nbsp;</td>
END
}

if (  defined($netsettings{'GRAY1_DEV'}) && $netsettings{'GRAY1_DEV'} ne 'disabled' )
{
print <<END;
     <td width='7%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wiogray.png' alt='$Lang::tr{'wio_gray'}' title='$Lang::tr{'wio_gray'}' />&nbsp;$Lang::tr{'wio_gray'}</td>
END
}
else
{
print <<END;
     <td width='7%' align='left' height='20'>&nbsp;</td>
END
}

if ( -e "$vpnpid" )
{
print <<END;
     <td width='7%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wiovpn.png' alt='$Lang::tr{'vpn'}' title='$Lang::tr{'vpn'}' />&nbsp;$Lang::tr{'vpn'}</td>
END
}
else
{
print <<END;
     <td width='7%' align='left' height='20'>&nbsp;</td>
END
}

if ( -e "$ovpnpid")
{
print <<END;
     <td width='13%' align='left' height='20'><img align='absmiddle' src='$imgstatic/wioovpn.png' alt='$Lang::tr{'ovpn'}' title='$Lang::tr{'ovpn'}' />&nbsp;$Lang::tr{'ovpn'}</td>
END
}
else
{
print <<END;
     <td width='13%' align='left' height='20'>&nbsp;</td>
END
}

print <<END;
	<td width='13%' align='left' height='20'>&nbsp;</td>
	<td width='18%' align='left' height='20'>&nbsp;</td>
</tr>
END

if ( -e "$ovpnpid" || -e "$vpnpid" )
{
print <<END;
<tr>
	<td width='15%' align='left' height='20'>&nbsp;</td>
	<td width='17%' align='left' height='20' colspan='2'><img align='absmiddle' src='$imgstatic/wion2n.png' alt='$Lang::tr{'wio_n2n'}' title='$Lang::tr{'wio_n2n'}' />&nbsp;$Lang::tr{'wio_n2n'}</td>
	<td width='17%' align='left' height='20' colspan='2'><img align='absmiddle' src='$imgstatic/wiorw.png' alt='$Lang::tr{'wio_rw'}' title='$Lang::tr{'wio_rw'}' />&nbsp;$Lang::tr{'wio_rw'}</td>
END
}
else
{
print <<END;
	<td width='15%' align='left' height='20'>&nbsp;</td>
	<td width='17%' align='left' height='20' colspan='2'>&nbsp</td>
	<td width='17%' align='left' height='20' colspan='2'>&nbsp</td>
END
}

print <<END;
	<td width='7%' align='left' height='20'  colspan='2'><img align='absmiddle' src='$imgstatic/wiograph.gif' alt='$Lang::tr{'wio_graphs'}' title='$Lang::tr{'wio_graphs'}' />&nbsp;$Lang::tr{'wio_graphs'}</td>
	<td width='13%' align='left' height='20'><img align='absmiddle' src='/images/reload.gif' alt='$Lang::tr{'wio_onoff_refresh'}' title='$Lang::tr{'wio_onoff_refresh'}' />&nbsp;$Lang::tr{'wio_onoff_refresh'}</td>
	<td width='13%' align='left' height='20'><img align='absmiddle' src='/images/add.gif' alt='$Lang::tr{'wio_add'}' title='$Lang::tr{'wio_add'}' />&nbsp;$Lang::tr{'wio_add'}</td>
	<td width='18%' align='left' height='20'><img align='absmiddle' src='$imgstatic/add_host.png' alt='$Lang::tr{'wio_no_add'}' title='$Lang::tr{'wio_no_add'}' />&nbsp;$Lang::tr{'wio_no_add'}</td>
</tr>
</table>
END

&Header::closebox();

}
else
{
}

############################################################################################################################
# add / modify client
############################################################################################################################

$checked{'EN'}{'on'} = ($wiosettings{'EN'} eq '' ) ? '' : "checked='checked'";

$checked{'DYNDNS'}{'off'} = '';
$checked{'DYNDNS'}{'on'} = '';
$checked{'DYNDNS'}{$wiosettings{'DYNDNS'}} = "checked='checked'";

$checked{'SENDEMAILON'}{'off'} = '';
$checked{'SENDEMAILON'}{'on'} = '';
$checked{'SENDEMAILON'}{$wiosettings{'SENDEMAILON'}} = "checked='checked'";

$checked{'SENDEMAILOFF'}{'off'} = '';
$checked{'SENDEMAILOFF'}{'on'} = '';
$checked{'SENDEMAILOFF'}{$wiosettings{'SENDEMAILOFF'}} = "checked='checked'";
	
&Header::openbox('100%', 'left', $Lang::tr{'wio_edit_settings'});

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
END

my $buttontext = $Lang::tr{'wio_client_add'};

if ($wiosettings{'ID'} ne '' && $wiosettings{'ACTION'} eq $Lang::tr{'edit'} || $errormessage ne '')
{
	$buttontext = $Lang::tr{'update'};

	$checked{'PINGMETHODE'}{'ip'} = '';
	$checked{'PINGMETHODE'}{'fqdn'} = '';
	$checked{'PINGMETHODE'}{$wiosettings{'PINGMETHODE'}} = "checked='checked'";
}
elsif ($wiosettings{'ID'} ne '' && $wiosettings{'ACTION'} eq $Lang::tr{'wio_add'})
{
	$checked{'PINGMETHODE'}{'ip'} = '';
	$checked{'PINGMETHODE'}{'fqdn'} = '';
	$checked{'PINGMETHODE'}{$wiosettings{'PINGMETHODE'}} = "checked='checked'";
}
else
{
	print "<tr><td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio add ip'}</b></td><td width='72%'>&nbsp;</td></tr><tr><td>&nbsp;</td></tr>";

	if ( $wiosettings{'PINGMETHODE'} eq '' )
	{
		$checked{'PINGMETHODE'}{'ip'} = "checked='checked'";
	}
	elsif ( $wiosettings{'PINGMETHODE'} eq 'ip' )
	{
		$checked{'PINGMETHODE'}{'ip'} = "checked='checked'";
	}
	else
	{
		$checked{'PINGMETHODE'}{'fqdn'} = "checked='checked'";	
	}
}

if ( $errormessage eq '' && $arp_msg eq '' && $wiosettings{'ID'} eq '' )
{

print <<END;
</table>
<table border='1' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr bgcolor='${Header::table1colour}'>
	<td width='3%' align='center' nowrap height='20'>$Lang::tr{'wio_id'}</td>
	<td width='15%' align='center' nowrap height='20'><a href='$ENV{'SCRIPT_NAME'}?IPADR'><b>$Lang::tr{'wio ipadress'}</b> $sortarrow1</a></td>
	<td width='20%' align='center' nowrap height='20'><a href='$ENV{'SCRIPT_NAME'}?HOST'><b>$Lang::tr{'wio name'}</b> $sortarrow2</a></td>
	<td colspan='2' width='10%' align='center' height='20' nowrap><b>$Lang::tr{'wio_ping_send'}</b></td>
	<td width='8%' align='center' height='20' nowrap><b>$Lang::tr{'wio_dyndns'}</b></td>
END
if ( $wiosettings{'MAIL'} ne 'on' )
{
	print"<td colspan='3' width='36%' align='center' height='20' nowrap><a href='$ENV{'SCRIPT_NAME'}?REMARK'><b>$Lang::tr{'remark'}</b> $sortarrow3</a></td>";
}
else
{
	print"<td colspan='2' width='10%' align='center' height='20' nowrap><b>$Lang::tr{'wio_sendemail'}</b></td>";
	print"<td width='26%' align='center' height='20' nowrap><a href='$ENV{'SCRIPT_NAME'}?REMARK'><b>$Lang::tr{'remark'}</b> $sortarrow3</a></td>";
}
print <<END;
	<td width='8%' colspan='4' align='center' height='20' nowrap><b>$Lang::tr{'action'}</b></td>
</tr>
<tr bgcolor='${Header::table1colour}'>
	<td width='3%' align='center' height='15'>&nbsp;</td>
	<td width='15%' align='center' height='15'>&nbsp;</td>
	<td width='20%' align='center' height='15'>&nbsp;</td>
	<td width='5%' align='center' height='15'><small>IP</small></td>
	<td width='5%' align='center' height='15'><small>FQDN</small></td>
	<td width='8%' align='center' height='15'>&nbsp;</td>
END
if ( $wiosettings{'MAIL'} ne 'on' )
{
	print"<td colspan='3' width='36%' align='center' height='15'>&nbsp;</td>";
}
else
{
	print"<td width='5%' align='center' height='15'><small>$Lang::tr{'wio_online'}</small></td>";
	print"<td width='5%' align='center' height='15'><small>$Lang::tr{'wio_offline'}</small></td>";
	print"<td width='26%' align='center' height='15'>&nbsp;</td>";
}
print <<END;
	<td width='8%' colspan='4' align='center' height='15'>&nbsp;</td>
</tr>
END

my $id = 0;

open(FILE ,"< $ipadrfile");
flock FILE, 2;
my @addressfile = <FILE>;
close(FILE);

foreach my $line (@addressfile)
{
	chomp($line);

	my @temp = split (/\,/, $line);

	my $gif = '';
	my $gdesc = ''; 
	my $dyndnsimg = '';
	my $dyndnsimgtxt = '';
	my $mailonimg = '';
	my $mailonimgtxt = '';
	my $mailoffimg = '';
	my $mailoffimgtxt = '';
	my $ipimg = '';
	my $ipimgtxt = '';
	my $fqdnimg = '';
	my $fqdnimgtxt = '';

	if ( $temp[2] eq 'on' )
	{
		$gif = 'on.gif';
		$gdesc = $Lang::tr{'wio_client_on'};
	}
	else
	{
		$gif = 'off.gif';
		$gdesc = $Lang::tr{'wio_client_off'};
	}

	if ( $temp[4] ne 'on' )
	{
		$dyndnsimg = 'off.gif';
		$dyndnsimgtxt = $Lang::tr{'wio_dyndns_off'};
	}
	else
	{
		$dyndnsimg = 'on.gif';
		$dyndnsimgtxt = $Lang::tr{'wio_dyndns_on'};
	}
	
	if ( $temp[5] ne 'on' )
	{
		$mailonimg = 'off.gif';
		$mailonimgtxt = $Lang::tr{'wio_mail_off'};
	}
	else
	{
		$mailonimg = 'on.gif';
		$mailonimgtxt = $Lang::tr{'wio_mail_on'};
	}

	if ( $temp[6] ne 'on' )
	{
		$mailoffimg = 'off.gif';
		$mailoffimgtxt = $Lang::tr{'wio_mail_off'};
	}
	else
	{
		$mailoffimg = 'on.gif';
		$mailoffimgtxt = $Lang::tr{'wio_mail_on'};
	}
	
	if ( $temp[7] eq 'ip' )
	{
		$ipimg = 'on.gif';
		$ipimgtxt = $Lang::tr{'wio_ip_on'};
	}
	else
	{
		$ipimg = 'off.gif';
		$ipimgtxt = $Lang::tr{'wio_ip_off'};
	}

	if ( $temp[7] eq 'fqdn' )
	{
		$fqdnimg = 'on.gif';
		$fqdnimgtxt = $Lang::tr{'wio_fqdn_on'};
	}
	else
	{
		$fqdnimg = 'off.gif';
		$fqdnimgtxt = $Lang::tr{'wio_fqdn_off'};
	}

if ($id % 2)
{
	print "<tr bgcolor='${Header::table1colour}'>"; 
}
else
{
	print "<tr>";
}

printf ("<td width='3%' align='center' nowrap> %02d</td>", $id+1);

print <<END;
<td align='center' height='20'>$temp[0]</td>
<td align='center' height='20'>$temp[1]</td>
<td align='center'><img align='absmiddle' src='/images/$ipimg' alt='$ipimgtxt' /></td>
<td align='center'><img align='absmiddle' src='/images/$fqdnimg' alt='$fqdnimgtxt' /></td>

<td align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<input type='hidden' name='ACTION' value='$Lang::tr{'enable disable dyndns'}' />
<input type='image' name='$Lang::tr{'enable disable dyndns'}' src='/images/$dyndnsimg' align='absmiddle' alt='$dyndnsimgtxt' title='$dyndnsimgtxt' />
<input type='hidden' name='ID' value='$id' /></form></td>
END

if ( $wiosettings{'MAIL'} eq 'off' && ( defined($temp[3]) && ( $temp[3] ne '' )) )
{
	print "<td colspan='3' align='center' nowrap>$temp[3]</td>";
}
elsif ( $wiosettings{'MAIL'} eq 'on' && $temp[3] eq '' )
{
	print "<td align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
	print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_mail_online'}' />";
	print "<input type='image' name='$Lang::tr{'wio_mail_online'}' src='/images/$mailonimg' align='absmiddle' alt='$mailonimgtxt' title='$mailonimgtxt' />";
	print "<input type='hidden' name='ID' value='$id' /></form></td>";

	print "<td align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
	print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_mail_offline'}' />";
	print "<input type='image' name='$Lang::tr{'wio_mail_offline'}' src='/images/$mailoffimg' align='absmiddle' alt='$mailoffimgtxt' title='$mailoffimgtxt' />";
	print "<input type='hidden' name='ID' value='$id' /></form></td>";

	print "<td align='center' nowrap>&nbsp;</td>";
}
elsif ( $wiosettings{'MAIL'} eq 'on' && ( defined($temp[3]) && ( $temp[3] ne '' )) )
{
	print "<td align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
	print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_mail_online'}' />";
	print "<input type='image' name='$Lang::tr{'wio_mail_online'}' src='/images/$mailonimg' align='absmiddle' alt='$mailonimgtxt' title='$mailonimgtxt' />";
	print "<input type='hidden' name='ID' value='$id' /></form></td>";

	print "<td align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
	print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_mail_offline'}' />";
	print "<input type='image' name='$Lang::tr{'wio_mail_offline'}' src='/images/$mailoffimg' align='absmiddle' alt='$mailoffimgtxt' title='$mailoffimgtxt' />";
	print "<input type='hidden' name='ID' value='$id' /></form></td>";

	print "<td align='center' nowrap>$temp[3]</td>";
}
else
{
	print "<td colspan='3' align='center' nowrap>&nbsp;</td>";
}

print <<END;
<td width='2%' align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<input type='hidden' name='ACTION' value='$Lang::tr{'enable disable client'}' />
<input type='image' name='$Lang::tr{'enable disable client'}' src='/images/$gif' align='absmiddle' alt='$gdesc' title='$gdesc' />
<input type='hidden' name='ID' value='$id' />
</form>
</td>
END

if ( defined($temp[4]) && ($temp[4] eq 'on') )
{
	print "<td width='2%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>";
	print "<input type='hidden' name='ACTION' value='$Lang::tr{'wio_dyndns_refresh'}' />";
	print "<input type='image' name='$Lang::tr{'wio_dyndns_refresh'}' src='/images/reload.gif' align='absmiddle' alt='$Lang::tr{'wio_dyndns_refresh'}' title='$Lang::tr{'wio_dyndns_refresh'}' />";
	print "<input type='hidden' name='ID' value='$id' /></form></td>";
}
else
{
	print "<td width='2%' align='center'>-</td>";
}

print <<END;
<td width='2%' align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<input type='hidden' name='ACTION' value='$Lang::tr{'edit'}' />
<input type='image' name='$Lang::tr{'edit'}' src='/images/edit.gif' align='absmiddle' alt='$Lang::tr{'edit'}' title='$Lang::tr{'edit'}' />
<input type='hidden' name='ID' value='$id' />
</form>
</td>

<td width='2%' align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>
<input type='hidden' name='ACTION' value='$Lang::tr{'remove'}' />
<input type='image' name='$Lang::tr{'remove'}' src='/images/delete.gif' align='absmiddle' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' />
<input type='hidden' name='ID' value='$id' />
</form>
</td>
</tr>
END

$id++;
}

print "</table>";

print <<END;
<table width='100%'>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

}
else
{
}

print <<END;
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ID' value='$wiosettings{'ID'}' />
<input type='hidden' name='ONLINE' value='$wiosettings{'ONLINE'}' />

<table width='100%'>
<tr>
	<td class='base'>$Lang::tr{'wio_client_enable'}</td>
	<td><input type='checkbox' name='EN' $checked{'EN'}{'on'} /></td>
	<td colspan='4'>&nbsp;</td>
</tr>
<tr>
	<td class='base'>$Lang::tr{'host ip'}:</td>
	<td><input type='text' name='IPADR' value='$wiosettings{'IPADR'}' SIZE='20' /></td>
	<td class='base'>$Lang::tr{'wio name'}:</td>
	<td><input type='text' name='HOST' value='$wiosettings{'HOST'}' SIZE='25' /></td>
	<td class='base'>$Lang::tr{'remark'}:&nbsp;<img align='absmiddle' src='/blob.gif' alt='*' /></td>
	<td><input type='text' name='REMARK' value='$wiosettings{'REMARK'}' size='30'></td>
</tr>
<tr>
	<td class='base'>$Lang::tr{'wio_ping_send'}:</td>
	<td align='left'>IP&nbsp;<input type='radio' name='PINGMETHODE' value='ip' $checked{'PINGMETHODE'}{'ip'} />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;FQDN&nbsp;<input type='radio' name='PINGMETHODE' value='fqdn' $checked{'PINGMETHODE'}{'fqdn'} /></td>
	<td class='base'>$Lang::tr{'wio_dyndns'}:</td>
	<td><input type='checkbox' name='DYNDNS' $checked{'DYNDNS'}{'on'} /></td>
END
if ( $wiosettings{'MAIL'} eq 'on' )
{
	print"<td class='base'>$Lang::tr{'wio_sendemail'}:</td>";
	print"<td><input type='checkbox' name='SENDEMAILON' $checked{'SENDEMAILON'}{'on'} />&nbsp;$Lang::tr{'wio_online'}&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='checkbox' name='SENDEMAILOFF' $checked{'SENDEMAILOFF'}{'on'} />&nbsp;$Lang::tr{'wio_offline'}</td>";
}
else
{
	print "<td>&nbsp;</td>";
	print "<td>&nbsp;</td>";
}
print <<END;
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
<table width='100%'>
<tr>
	<td width='30%'>&nbsp;</td>
	<td width='14%'>&nbsp;</td>
END

if ( $buttontext eq $Lang::tr{'update'}  && $errormessage ne '' )
{
	print "<td class='base' width='14%' align='center'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_client_add'}' /><input type='submit' name='SUBMIT' value='$buttontext' /></form></td>";
}
else
{
	print "<td class='base' width='14%' align='center'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_client_add'}' /><input type='submit'  name='SUBMIT' value='$buttontext' /></form></td>";
}

if ( $buttontext eq $Lang::tr{'update'} )
{
	print "<td class='base' width='14%' align='center'>&nbsp;</td>";
	print "<td class='base' width='14%' align='center'>&nbsp;</td>";
}
elsif ( $buttontext eq $Lang::tr{'wio_client_add'} && $errormessage ne '' )
{
	print "<td class='base' width='14%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_back'}' /></form></td>";
	print "<td class='base' width='14%' align='center'>&nbsp;</td>";
}
elsif ( $buttontext eq $Lang::tr{'wio_client_add'} && $arp_msg ne '' )
{
	print "<td class='base' width='14%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_client_clear'}' /></form></td>";
	print "<td class='base' width='14%' align='center'>&nbsp;</td>";
}
else
{
	print "<td class='base' width='14%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_client_clear'}' /></form></td>";
	print "<td class='base' width='14%' align='center'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'><input type='submit' name='ACTION' value='$Lang::tr{'wio_delete'}' /></form></td>";
}

print <<END;
</tr>
</table>
END

if ( $wiosettings{'ID'} eq '' && $errormessage eq '' )
{

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td colspan='2' height='35'><hr></td>
</tr>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_import_file'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>

<table width='100%' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<form method="post" action="/cgi-bin/wio.cgi" enctype="multipart/form-data">
	<td width='37%' align='right'>$Lang::tr{'wio_import_csv'}&nbsp;</td>
	<td width='35%' align='center'><input type='file' name='CSVFILE' size='35' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_import'}1' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_import'}' /></td>
	</form>
</tr>
<tr>
	<td colspan='3'  height='25'></td>
</tr>
</table>
<table width='100%' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<form method="post" action="/cgi-bin/wio.cgi" enctype="multipart/form-data">
	<td width='37%' ALIGN='right'>$Lang::tr{'wio_import_hosts'}&nbsp;</td>
	<td width='35%' ALIGN='left'>&nbsp;</td>
	<td width='28%' ALIGN='center'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_import'}2' /><input type='SUBMIT' name='SUBMIT' value='$Lang::tr{'wio_import'}' /></td>
	</form>
</tr>
<tr>
	<td colspan='3'  height='25'></td>
</tr>
</table>
<table width='100%' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<form method="post" action="/cgi-bin/wio.cgi" enctype="multipart/form-data">
	<td width='37%' ALIGN='right'>$Lang::tr{'wio_import_fixleases'}&nbsp;</td>
	<td width='35%' ALIGN='left'>&nbsp;</td>
	<td width='28%' ALIGN='center'><input type='hidden' name='ACTION' value='$Lang::tr{'wio_import'}3' /><input type='SUBMIT' name='SUBMIT' value='$Lang::tr{'wio_import'}' /></td>
	</form>
</tr>
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

print <<END;
<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td colspan='2' height='35'><hr></td>
</tr>
<tr>
	<td width='28%' bgcolor='${Header::table1colour}' align='left' height='20'><b>&nbsp;$Lang::tr{'wio_net_scan'}</b></td>
	<td width='72%'>&nbsp;</td>
</tr>
<tr>
	<td width='28%'>&nbsp;</td>
	<td width='72%'>&nbsp;</td>
</tr>
</table>

<table border='0' width='100%' bordercolor='#000000' cellspacing='0' cellpadding='0' style='border-collapse: collapse'>
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='${Header::colourgreen}'><b>$Lang::tr{'wio_net_scan_green'}</b></font> $Lang::tr{'wio_net_scan_vl'} $Lang::tr{'wio_net_scan_l'}<font color='${Header::colourgreen}'><b> $netsettings{'GREEN_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='GREENIPLOW' value='$wiosettings{'GREENIPLOW'}' SIZE='14' STYLE='background-color:#339933; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='GREENIPHIGH' value='$wiosettings{'GREENIPHIGH'}' SIZE='14' STYLE='background-color:#339933; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_green' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

if ( defined($netsettings{'BLUE_DEV'}) && defined($netsettings{'BLUE_ADDRESS'}) )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='${Header::colourblue}'><b>$Lang::tr{'wio_net_scan_blue'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='${Header::colourblue}'><b> $netsettings{'BLUE_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='BLUEIPLOW' value='$wiosettings{'BLUEIPLOW'}' SIZE='14' STYLE='background-color:#333399; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='BLUEIPHIGH' value='$wiosettings{'BLUEIPHIGH'}' SIZE='14' STYLE='background-color:#333399; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_blue' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

if ( defined($netsettings{'ORANGE_DEV'}) && defined($netsettings{'ORANGE_ADDRESS'}) )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='${Header::colourorange}'><b>$Lang::tr{'wio_net_scan_orange'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='${Header::colourorange}'><b> $netsettings{'ORANGE_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='ORANGEIPLOW' value='$wiosettings{'ORANGEIPLOW'}' SIZE='14' STYLE='background-color:#FF9933; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='ORANGEIPHIGH' value='$wiosettings{'ORANGEIPHIGH'}' SIZE='14' STYLE='background-color:#FF9933; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_orange' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

if ( defined($netsettings{'GRAY1_DEV'}) && $netsettings{'GRAY1_DEV'} ne 'disabled' )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='#585858'><b>$Lang::tr{'wio_net_scan_gray'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='#585858'><b> $netsettings{'GRAY1_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='GRAY1IPLOW' value='$wiosettings{'GRAY1IPLOW'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='GRAY1IPHIGH' value='$wiosettings{'GRAY1IPHIGH'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_gray1' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

if ( defined($netsettings{'GRAY2_DEV'}) && $netsettings{'GRAY2_DEV'} ne 'disabled' )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='#585858'><b>$Lang::tr{'wio_net_scan_gray'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='#585858'><b> $netsettings{'GRAY2_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='GRAY2IPLOW' value='$wiosettings{'GRAY2IPLOW'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='GRAY2IPHIGH' value='$wiosettings{'GRAY2IPHIGH'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_gray2' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

if ( defined($netsettings{'GRAY3_DEV'}) && $netsettings{'GRAY3_DEV'} ne 'disabled' )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='#585858'><b>$Lang::tr{'wio_net_scan_gray'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='#585858'><b> $netsettings{'GRAY3_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='GRAY3IPLOW' value='$wiosettings{'GRAY3IPLOW'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='GRAY3IPHIGH' value='$wiosettings{'GRAY3IPHIGH'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_gray3' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

if ( defined($netsettings{'GRAY4_DEV'}) && $netsettings{'GRAY4_DEV'} ne 'disabled' )
{

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
<tr>
	<td width='37%' align='right'><form method='post' action='$ENV{'SCRIPT_NAME'}' enctype='multipart/form-data'>$Lang::tr{'wio_net_scan_vl'} <font color='#585858'><b>$Lang::tr{'wio_net_scan_gray'}</b></font> $Lang::tr{'wio_net_scan_l'}<font color='#585858'><b> $netsettings{'GRAY4_DEV'} </b></font>$Lang::tr{'wio_net_scan_r'}</td>
	<td width='35%' align='center'><input type='text' name='GRAY4IPLOW' value='$wiosettings{'GRAY4IPLOW'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /> - <input type='text' name='GRAY4IPHIGH' value='$wiosettings{'GRAY4IPHIGH'}' SIZE='14' STYLE='background-color:#585858; text-align: center; color:white; font-weight: bold' /></td>
	<td width='28%' align='center'><input type='hidden' name='ACTION' value='wio_run_gray4' /><input type='submit' name='SUBMIT' value='$Lang::tr{'wio_net_scan_run'}'></form></td>
</tr>
END

}

print <<END;
<tr>
	<td>&nbsp;</td>
</tr>
</table>
END

}
else
{
}

print <<END;
<table width='100%'>
<tr>
	<td colspan='2'>&nbsp;</td>
</tr>
</table>
<table width='100%' border='0' align='left' cellspacing='0' cellpadding='0'>
<tr>
	<td width='12%' align='left' height='20'><b>&nbsp;$Lang::tr{'legend'}:</b></td>
	<td width='24%' align='left' height='20'><img align='absmiddle' src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
	<td width='11%' align='left' height='20'><img align='absmiddle' src='/images/on.gif' alt='$Lang::tr{'click to disable'}' />&nbsp;$Lang::tr{'wio click to disable'}</td>
	<td width='11%' align='left' height='20'><img align='absmiddle' src='/images/off.gif' alt='$Lang::tr{'click to enable'}' />&nbsp;$Lang::tr{'wio click to enable'}</td>
	<td width='20%' align='left' height='20'><img align='absmiddle' src='/images/reload.gif' alt='$Lang::tr{'wio_dyndns_refresh'}' />&nbsp;$Lang::tr{'wio_dyndns_refresh'}</td>
	<td width='11%' align='left' height='20'><img align='absmiddle' src='/images/edit.gif' alt='$Lang::tr{'edit'}' />&nbsp;$Lang::tr{'edit'}</td>
	<td width='11%' align='left' height='20'><img align='absmiddle' src='/images/delete.gif' alt='$Lang::tr{'remove'}' />&nbsp;$Lang::tr{'remove'}</td>
</tr>
</table>
END

&Header::closebox();
}

&Header::closebigbox();
&Header::closepage();

############################################################################################################################

sub SortDataFile
{
	our %entries = ();
    
	sub sortips
	{
		my $qs = '';
		
		if (rindex ($wiosettings{'SORT'},'Rev') != -1)
		{
			$qs = substr ($wiosettings{'SORT'},0,length($wiosettings{'SORT'})-3);
			
			if ($qs eq 'IPADR')
			{
				my @a = split (/\./,$entries{$a}->{$qs});
				my @b = split (/\./,$entries{$b}->{$qs});
				($b[0]<=>$a[0]) ||
				($b[1]<=>$a[1]) ||
				($b[2]<=>$a[2]) ||
				($b[3]<=>$a[3]);
			}
			else
			{
				$entries{$b}->{$qs} cmp $entries{$a}->{$qs};
			}
		}
		else 
		{
			$qs = $wiosettings{'SORT'};
			
			if ($qs eq 'IPADR')
			{
				my @a = split (/\./,$entries{$a}->{$qs});
				my @b = split (/\./,$entries{$b}->{$qs});
				($a[0]<=>$b[0]) ||
				($a[1]<=>$b[1]) ||
				($a[2]<=>$b[2]) ||
				($a[3]<=>$b[3]);
			}
			else
			{
				$entries{$a}->{$qs} cmp $entries{$b}->{$qs};
			}
		}
	}
	
	my $id = 0;
	
	foreach my $line (@current)
	{
		chomp( $line);
		my @temp = split (',', $line);
	
		my @record = ('KEY',$id++,'IPADR',$temp[0],'HOST',$temp[1],'EN',$temp[2],'REMARK',$temp[3],'DYNDNS',$temp[4],'SENDEMAILON',$temp[5],'SENDEMAILOFF',$temp[6],'PINGMETHODE',$temp[7],'ONLINE',$temp[8]);
		my $record = {};
		%{$record} = @record; 
		$entries{$record->{KEY}} = $record;
	}

	open(FILE, "> $ipadrfile");
	flock FILE, 2;

	foreach my $entry (sort sortips keys %entries)
	{
		print FILE "$entries{$entry}->{IPADR},$entries{$entry}->{HOST},$entries{$entry}->{EN},$entries{$entry}->{REMARK},$entries{$entry}->{DYNDNS},$entries{$entry}->{SENDEMAILON},$entries{$entry}->{SENDEMAILOFF},$entries{$entry}->{PINGMETHODE},$entries{$entry}->{ONLINE}\n";
	}

	close(FILE);

	open (FILE, "$ipadrfile");
	flock FILE, 2;
	@current = <FILE>;
	close (FILE);
}
